"""
Module dedicated to perform the data transformation for the NVD3 charts
"""

from epyk.core.js import JsUtils


class JsNVD3Links(object):
  def __init__(self, data, js_src, data_schema=None, profile=False):
    self._js_src, self._data_schema, self.profile, self._data = js_src, data_schema, profile, data

  def __register_records_fnc(self, fnc_name, fnc_def, fnc_pmts=None, profile=False):
    """
    This function will attach to the report object only the javascript functions used during the report

    :param fnc_name: A String with the Javascript function name to be defined
    :param fnc_def: A String with the Javascript function content
    :param fnc_pmts: A list of parameters
    :param profile: A boolean flag to activate the framework profiling

    :return:
    """
    fnc_pmts = ["data"] + (fnc_pmts or [])
    if not fnc_name in self._js_src.get('js', {}).get('functions', {}):
      if profile:
        content = "var result = []; %s;return result" % JsUtils.cleanFncs(fnc_def)
      else:
        content = "var result = []; %s;return result" % JsUtils.cleanFncs(fnc_def)
      self._js_src.setdefault('js', {}).setdefault('functions', {})[fnc_name] = {'content': content, 'pmt': fnc_pmts}

  def pie(self, y_columns, x_axis, profile=False):
    """
    Data Conversion to a NVD3 pie chart object.

    This entry point will register the Javascript function and then use it on the javascript side using the parameters
    passed in this function. Some basic controls are done on the Python side during the transpilation

    Example

    :param y_columns: A python list with the columns in the records
    :param x_axis: A string with a column name from the records
    :param profile: A boolean flag to activate the framework profiling

    :return:
    """
    if not isinstance(y_columns, list):
      y_columns = [y_columns]
    fnc_name = JsNVD3Pie.__name__
    x_axis = JsUtils.jsConvertData(x_axis, None)
    self.__register_records_fnc(fnc_name, JsNVD3Pie.content, list(JsNVD3Pie.pmts), profile | self.profile)
    self._data_schema['out'] = "%s(%%s, %s, %s)" % (fnc_name, y_columns, x_axis)
    return self._data

  def bar(self, y_columns, x_axis, profile=False):
    """
    Data Conversion to a NVD3 bar chart object.

    This entry point will register the Javascript function and then use it on the javascript side using the parameters
    passed in this function. Some basic controls are done on the Python side during the transpilation

    Example
    record = jsObj.data.records([{"B": 'RR3', "A": 0}, {"B": 'RR5', "A": 3}, {"B": 'RR3', "A": 2}])
    record.f.count("B").o.nvd3.pie("count", "column")

    :param y_columns: A python list with the columns in the records
    :param x_axis: A string with a column name from the records

    :return:
    """
    if not isinstance(y_columns, list):
      y_columns = [y_columns]
    fnc_name = JsNVD3Bar.__name__
    x_axis = JsUtils.jsConvertData(x_axis, None)
    self.__register_records_fnc(fnc_name, JsNVD3Bar.content, list(JsNVD3Bar.pmts), profile | self.profile)
    self._data_schema['out'] = "%s(%%s, %s, %s)" % (fnc_name, y_columns, x_axis)
    return self._data

  def line(self, y_columns, x_axis, profile=False):
    """

    :param y_columns:
    :param x_axis:
    :param profile:

    """
    return self.bar(y_columns, x_axis, profile)

  def sunburst(self, y_columns, x_axis, profile=False):
    """
    Data Conversion to a NVD3 Sunburst chart object.

    This entry point will register the Javascript function and then use it on the javascript side using the parameters
    passed in this function. Some basic controls are done on the Python side during the transpilation

    Example

    :param y_columns: A python list with the columns in the records
    :param x_axis: A string with a column name from the records

    :return:
    """
    if not isinstance(y_columns, list):
      y_columns = [y_columns]
    fnc_name = JsNVD3Bar.__name__
    x_axis = JsUtils.jsConvertData(x_axis, None)
    self.__register_records_fnc(fnc_name, JsNVD3Sunburst.content, list(JsNVD3Sunburst.pmts), profile | self.profile)
    self._data_schema['out'] = "%s(%%s, %s, %s)" % (fnc_name, y_columns, x_axis)


# --------------------------------------------------------------------------------------------------------------------
#                               DATA TRANSFORMATION JAVASCRIPT DEFINITION
# --------------------------------------------------------------------------------------------------------------------
class JsNVD3Pie(object):
  pmts = ("seriesNames", "xAxis")
  content = '''
    var temp = {}; var labels = {};
    data.forEach(function(rec){ 
      if(!(rec[xAxis] in temp)){temp[rec[xAxis]] = {}};
      seriesNames.forEach(function(name){
        labels[name] = true; if(rec[name] !== undefined) {if (!(name in temp[rec[xAxis]])){temp[rec[xAxis]][name] = rec[name]} else {temp[rec[xAxis]][name] += rec[name]}}  }) ;
    });
    var labels = Object.keys(labels); result = [];
    for(var series in temp){
      var values = {y: 0, x: series};
      labels.forEach(function(label){
        if(temp[series][label] !== undefined){values.y = temp[series][label]}});
      result.push(values)}
    '''


class JsNVD3Bar(object):
  pmts = ("seriesNames", "xAxis")
  content = '''
    var temp = {}; var labels = []; var uniqLabels = {};
    seriesNames.forEach(function(series){temp[series] = {}}) ;
    data.forEach(function(rec){ 
      seriesNames.forEach(function(name){
        if(rec[name] !== undefined){
          if (!(rec[xAxis] in uniqLabels)){labels.push(rec[xAxis]); uniqLabels[rec[xAxis]] = true};
          temp[name][rec[xAxis]] = rec[name]}})
    });
    seriesNames.forEach(function(series){
      dataSet = {key: series, values: [], labels: labels};
      labels.forEach(function(x, i){
        var value = temp[series][x]; 
        if (isNaN(value)) { value = null};
        if (value !== undefined) {dataSet.values.push({y: value, x: i, label: x})}
      }); result.push(dataSet)})
    '''


class JsNVD3Sunburst(object):
  pmts = ("seriesNames", "xAxis")
  content = '''
    var result = [{name: xAxis, children: []}]; var sizeTree = seriesNames.length-1;
    data.forEach(function(rec){
      var path = []; var tmpResultLevel = result[0].children; var branchVal = 0;
      seriesNames.forEach(function(s, i){
        var treeLevel = -1; 
        tmpResultLevel.forEach(function(l, j){if(l.name == rec[s]){treeLevel = j}});
        if(i == sizeTree){
          if(treeLevel >= 0){
            tmpResultLevel[treeLevel].size += rec[xAxis]}else{tmpResultLevel.push({name: rec[s], size: rec[xAxis]})}
        }else{
          if(treeLevel < 0 ){
            tmpResultLevel.push({name: rec[s], children: []}); treeLevel = tmpResultLevel.length - 1};
            tmpResultLevel = tmpResultLevel[treeLevel].children}
      })})'''




from epyk.core.css.catalogs import Catalog

from epyk.core.css.styles.classes import CssStyleDropdown


class CatalogDropDown(Catalog.CatalogGroup):

  def base(self):
    """
    Description:
    ------------
    """
    return self._set_class(CssStyleDropdown.CssDropDownSubMenu)

  def menu(self):
    """
    Description:
    ------------
    """
    return self._set_class(CssStyleDropdown.CssDropDownMenu)

  def menu_li(self):
    """
    Description:
    ------------
    """
    return self._set_class(CssStyleDropdown.CssDropDownMenuHLi)

  def menu_after(self):
    """
    Description:
    ------------
    """
    return self._set_class(CssStyleDropdown.CssDropDownAfterMenu)

  def menu_hover(self):
    """
    Description:
    ------------
    """
    return self._set_class(CssStyleDropdown.CssDropDownMenuHoverAAfter)

  def menu_pull_left(self):
    """
    Description:
    ------------
    """
    return self._set_class(CssStyleDropdown.CssDropDownSubMenuPullLeft)





from epyk.core.data import DataClass
from epyk.core.data import DataEnum


class EnumStyleOptions(DataEnum):

  def api(self):
    """
    Description:
    -----------
    Selection can only be performed via the API

    Related Pages:

			https://datatables.net/reference/option/select.style
    """
    return self.set()

  def single(self):
    """
    Description:
    -----------
    Only a single item can be selected, any other selected items will be automatically deselected when a new item is selected

    Related Pages:

			https://datatables.net/reference/option/select.style
    """
    return self.set()

  def multi(self):
    """
    Description:
    -----------
    Multiple items can be selected. Selection is performed by simply clicking on the items to be selected

    Related Pages:

			https://datatables.net/reference/option/select.style
    """
    return self.set()

  def os(self):
    """
    Description:
    -----------
    Operating System (OS) style selection. This is the most comprehensive option and provides complex behaviours such as ctrl/cmd clicking to select / deselect individual items, shift clicking to select ranges and an unmodified click to select a single item.

    Related Pages:

			https://datatables.net/reference/option/select.style
    """
    return self.set()

  def multi_shift(self):
    """
    Description:
    -----------
    a hybrid between the os style and multi, allowing easy multi-row selection without immediate de-selection when clicking on a row.

    Related Pages:

			https://datatables.net/reference/option/select.style
    """
    return self.set("multi+shift")


class Select(DataClass):

  def activate(self):
    """
    Enable and configure the Scroller extension for DataTables

    https://datatables.net/reference/option/scroller
    """
    self.info = False
    self.items = 'row'
    return self

  @property
  def info(self):
    """
    Enable / disable the display for item selection information in the table summary

    https://datatables.net/reference/option/select.info
    """
    return self._attrs["info"]

  @info.setter
  def info(self, val):
    self._attrs["info"] = val

  @property
  def blurable(self):
    """
    Description:
    -----------
    Indicate if the selected items will be removed when clicking outside of the table

    Related Pages:

			https://datatables.net/reference/option/select.blurable
    """
    return self._attrs["blurable"]

  @blurable.setter
  def blurable(self, val):
    self._attrs["blurable"] = val

  @property
  def items(self):
    """
    Description:
    -----------
    Set which table items to select (rows, columns or cells).

    Related Pages:

			https://datatables.net/reference/option/select.items
    """
    return self._attrs["items"]

  @items.setter
  def items(self, val):
    self._attrs["items"] = val

  @property
  def style(self):
    """
    Description:
    -----------
    Set the selection style for end user interaction with the table.

    Related Pages:

			https://datatables.net/reference/option/select.style
    """
    return self._attrs["style"]

  @style.setter
  def style(self, val):
    self._attrs["style"] = val

  @property
  def toggleable(self):
    """
    Description:
    -----------
    Disable the deselection of selected rows when clicked.

    Related Pages:

			https://datatables.net/reference/option/select.toggleable
    """
    return self._attrs["toggleable"]

  @toggleable.setter
  def toggleable(self, val):
    self._attrs["toggleable"] = val



"""
Wrapper to the Javascript Socket module

Related Pages:

	ttps://www.tutorialspoint.com/html5/html5_websocket.htm
  https://blog.miguelgrinberg.com/post/easy-websockets-with-flask-and-gevent
"""


from epyk.core.js import JsUtils


class JsWebSocket(object):
  def __init__(self, htmlCode, url, protocol, service, context=None):
    """

    :param src:
    :param htmlCode:
    :param url:
    :param protocol:
    :param service:
    """
    wsId = JsUtils.jsConvert(htmlCode)
    self._context = context if context is not None else {}
    self._context.setdefault("modules", set()).add('socket.io')
    self._context.setdefault('onReady', []).append("%s = io.connect('%s:%s/%s')" % (wsId, url, protocol, service))
    self._js, self.wsId = [], wsId

  def send(self, message):
    """

    :return:
    """

  def close(self):
    """

    :return:
    """

  def open(self, jsFncs):
    """

    :param jsFncs:
    :return:
    """

  def message(self, jsFncs):
    """

    :param jsFncs:
    :return:
    """





from epyk.core.css.styles import GrpCls
from epyk.core.css import Classes
from epyk.core.css.styles.attributes import Commons


class ContentTable(GrpCls.ClassHtml):
  @property
  def css(self):
    """
    Description:
    ------------
    Property to the underlying CSS definition to be added to the style HTML tag of a component

    :rtype: Commons
    """
    if self._css_struct is None:
      self._css_struct = Commons(self.htmlObj)
    return self._css_struct

  @property
  def css_class(self):
    """
    The internal class used to put a custom Style to this object.
    Only 1 CSS class can be added to an HTML object

    :rtype: Classes.CatalogInput.CatalogInput
    """
    if self._css_class is None:
      self._css_class = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['main'], html_id=self.htmlObj.htmlId).table_content()
    return self._css_class




from epyk.core.css.styles import GrpCls
from epyk.core.css import Classes


class DataTableThemes(object):

  def __init__(self, classlist):
    self.classlist = classlist

  def cell_border(self):
    """

    https://datatables.net/examples/styling/cell-border.html

    cell-border

    :return:
    """
    self.classlist.add("cell-border")
    return self

  def compact(self):
    """
    Reduce the amount of white-space the default styling for the DataTable uses, increasing the information density on screen, as shown below.
    Note that this style requires DataTables 1.10.1 or newer.

    https://datatables.net/manual/styling/classes
    """
    self.classlist.add("compact")
    return self

  def display_compact(self):
    """

    https://datatables.net/examples/styling/compact.html
    :return:
    """
    self.classlist.add("display")
    self.classlist.add("compact")
    return self

  def hover(self):
    """

    https://datatables.net/examples/styling/hover.html

    hover

    :return:
    """
    self.classlist.add("hover")
    return self

  def order_column(self):
    """
    Highlight the ordering column

    https://datatables.net/examples/styling/order-column.html
    """
    self.classlist.add("order-column")
    return self

  def nowrap(self):
    """
    Disable line wrapping of content in the table cells, so the text will always appear on one line.
    Note that this style requires DataTables 1.10.1 or newer.

    https://datatables.net/manual/styling/classes
    """
    self.classlist.add("nowrap")
    return self

  def row_border(self):
    """
    Border on the rows only

    https://datatables.net/examples/styling/row-border.html
    """
    self.classlist.add("row-border")
    return self

  def stripe(self):
    """
    Row striping.

    https://datatables.net/examples/styling/stripe.html
    """
    self.classlist.add("stripe")
    return self

  def bootstrap(self):
    """

    https://datatables.net/examples/styling/bootstrap4.html
    """
    self.classlist.add("table")
    self.classlist.add("table-striped")
    self.classlist.add("table-bordered")
    return self


class Datatable(GrpCls.ClassHtml):

  def __init__(self, htmlObj):
    super(Datatable, self).__init__(htmlObj)
    self._css_datatable, self._css_datatable_header, self._css_datatable_row_odd = None, None, None
    self._css_datatable_row_even, self._css_datatable_footer = None, None
    self.classList['main'].add(self.cls_datatable)
    self.classList['main'].add(self.cls_datatable_header)
    self.classList['main'].add(self.cls_datatable_odd)
    self.classList['main'].add(self.cls_datatable_even)
    self.classList['main'].add(self.cls_datatable_footer)

  @property
  def themes(self):
    """
    Add the predefined themes in the javascript library

    https://datatables.net/examples/styling/index.html
    """
    return DataTableThemes(self.classList['main'])

  @property
  def cls_datatable(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_datatable is None:
      self._css_datatable = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['main']).datatable()
    return self._css_datatable

  @property
  def cls_datatable_header(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_datatable_header is None:
      self._css_datatable_header = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['main']).datatable_header()
    return self._css_datatable_header

  @property
  def cls_datatable_odd(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_datatable_row_odd is None:
      self._css_datatable_row_odd = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['main']).datatable_odd()
    return self._css_datatable_row_odd

  @property
  def cls_datatable_even(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_datatable_row_even is None:
      self._css_datatable_row_even = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['main']).datatable_even()
    return self._css_datatable_row_even

  @property
  def cls_datatable_footer(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_datatable_footer is None:
      self._css_datatable_footer = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['main']).datatable_footer()
    return self._css_datatable_footer


class Tabulator(GrpCls.ClassHtml):

  def __init__(self, htmlObj):
    super(Tabulator, self).__init__(htmlObj)
    self._css_tabulator, self._css_tabulator_row, self._css_tabulator_header = None, None, None
    self._css_tabulator_even_row, self._css_tabulator_cell, self._css_tabulator_headers = None, None, None
    self._css_tabulator_col, self._css_tabulator_col_content, self._css_tabulator_selected = None, None, None
    self._css_tb_odd_row, self._css_tb_groups, self._css_tb_footer = None, None, None
    self._css_tb_footer_pg, self._css_tb_tree, self._css_tb_tree_exp = None, None, None
    self.classList['main'].add(self.cls_tabulator)
    self.classList['other'].add(self.cls_tabulator_row)
    self.classList['other'].add(self.cls_tabulator_header)
    self.classList['other'].add(self.cls_tabulator_even_row)
    self.classList['other'].add(self.cls_tabulator_cell)
    self.classList['other'].add(self.cls_tabulator_headers)
    self.classList['other'].add(self.cls_tabulator_col)
    self.classList['other'].add(self._css_tabulator_col_content)
    self.classList['other'].add(self.cls_tabulator_selected)
    self.classList['other'].add(self.cls_tb_even_row)
    self.classList['other'].add(self.cls_tb_groups)
    self.classList['other'].add(self.cls_tb_footer)
    self.classList['other'].add(self.cls_tb_footer_pg)
    self.classList['other'].add(self.cls_tb_tree)
    self.classList['other'].add(self.cls_tb_tree_exp)

  @property
  def cls_tabulator(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator is None:
      self._css_tabulator = Classes.CatalogTable.CatalogTable(self.htmlObj._report,
                                                                  self.classList['main']).tabulator()
    return self._css_tabulator

  @property
  def cls_tabulator_row(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator_row is None:
      self._css_tabulator_row = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_rows()
    return self._css_tabulator_row

  @property
  def cls_tabulator_cell(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator_cell is None:
      self._css_tabulator_cell = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_cell()
    return self._css_tabulator_cell

  @property
  def cls_tabulator_col(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator_col is None:
      self._css_tabulator_col = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_col()
    return self._css_tabulator_col

  @property
  def cls_tabulator_col_content(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator_col_content is None:
      self._css_tabulator_col_content = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_col_content()
    return self._css_tabulator_col_content

  @property
  def cls_tabulator_selected(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator_selected is None:
      self._css_tabulator_selected = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList[
        'other']).tabulator_selected()
    return self._css_tabulator_selected

  @property
  def cls_tabulator_even_row(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator_even_row is None:
      self._css_tabulator_even_row = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_even_rows()
    return self._css_tabulator_even_row

  @property
  def cls_tb_even_row(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tb_odd_row is None:
      self._css_tb_odd_row = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_odd_rows()
    return self._css_tb_odd_row

  @property
  def cls_tb_groups(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tb_groups is None:
      self._css_tb_groups = Classes.CatalogTable.CatalogTable(self.htmlObj._report,
                                                               self.classList['other']).tabulator_groups()
    return self._css_tb_groups

  @property
  def cls_tb_footer(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tb_footer is None:
      self._css_tb_footer = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_footer()
    return self._css_tb_footer

  @property
  def cls_tb_footer_pg(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tb_footer_pg is None:
      self._css_tb_footer_pg = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_footer_pagination()
    return self._css_tb_footer_pg

  @property
  def cls_tb_tree(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tb_tree is None:
      self._css_tb_tree = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_tree_control()
    return self._css_tb_tree

  @property
  def cls_tb_tree_exp(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tb_tree_exp is None:
      self._css_tb_tree_exp = Classes.CatalogTable.CatalogTable(self.htmlObj._report,
                                                            self.classList['other']).tabulator_tree_control_expand()
    return self._css_tb_tree_exp

  @property
  def cls_tabulator_header(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator_header is None:
      self._css_tabulator_header = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_header()
    return self._css_tabulator_header

  @property
  def cls_tabulator_headers(self):
    """

    :rtype: Classes.CatalogTable.CatalogTable
    """
    if self._css_tabulator_headers is None:
      self._css_tabulator_headers = Classes.CatalogTable.CatalogTable(self.htmlObj._report, self.classList['other']).tabulator_headers()
    return self._css_tabulator_headers





PLOTLY_POLAR = r'https://raw.githubusercontent.com/plotly/datasets/master/polar_dataset.csv'
PLOTLY_POINTS = r'https://raw.githubusercontent.com/plotly/datasets/master/3d-line1.csv'

data_line_3d = r'https://raw.githubusercontent.com/plotly/datasets/master/_3d-line-plot.csv'
data_mesh_3d = r'https://raw.githubusercontent.com/plotly/datasets/master/mesh_dataset.txt'

PLOTLY_APPLE_PRICES = r'https://raw.githubusercontent.com/plotly/datasets/master/finance-charts-apple.csv'

GEO_US_CITIES = 'https://raw.githubusercontent.com/plotly/datasets/master/2014_us_cities.csv'
GEO_US_RAINS = 'https://raw.githubusercontent.com/plotly/datasets/master/2015_06_30_precipitation.csv'
GEO_EARTHQUAKES = 'https://raw.githubusercontent.com/plotly/datasets/master/earthquakes-23k.csv'

GEO_COVID19_COUNTRIES = "'https://covid.ourworldindata.org/data/total_cases.csv'"

AIRPORT_TRAFFIC = 'https://raw.githubusercontent.com/plotly/datasets/master/2011_february_us_airport_traffic.csv'
PLOTLY_WEBGL_POLAR = 'https://raw.githubusercontent.com/plotly/datasets/master/hobbs-pearson-trials.csv'

BANKING_DATA = 'https://raw.githubusercontent.com/juliencohensolal/BankMarketing/master/rawData/bank-additional-full.csv'


#####################################################################################################################
# AG GRID
# website: https://github.com/ag-grid
#
OLYMPIC_WINNERS_SMALL = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/olympicWinnersSmall.json'
OLYMPIC_WINNERS_FULL = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/olympicWinners.json'
MONTLY_SALES = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/sample-data/monthlySales.json'
STOCKS = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/stocks.json'
BLOG_OBJECT = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/blogs.json'
WEATHER_ENGLAND = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/weather_se_england.json'
AGGRID_OBJECTS_NESTED = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/feature-overlays.json'

SP500W_DATA = 'https://raw.githubusercontent.com/ag-grid/ag-grid-charts-example/master/data/sp500w.csv'
GE_SHARE_DATA = 'https://raw.githubusercontent.com/ag-grid/ag-grid-charts-example/master/data/GE.csv'
MSFT_DATA = 'https://raw.githubusercontent.com/ag-grid/ag-grid-charts-example/master/data/MSFT.csv'
BODY_DATA = 'https://raw.githubusercontent.com/ag-grid/ag-grid-charts-example/master/data/body-data.csv'


# https://bl.ocks.org/d3noob
DC_QUAKES = 'http://bl.ocks.org/d3noob/raw/6077996/45425ba06fc3c30ed3c7492d1042352d3dad0f18/quakes.csv'

#https://earthquake.usgs.gov/earthquakes/feed/v1.0/csv.php
DATA_EARTHQUAKE = 'https://earthquake.usgs.gov/earthquakes/feed/v1.0/summary/all_week.csv'
WORLD_INDEX = 'https://visjs.github.io/vis-graph3d/examples/graph3d/rotateAxisLabels/world_index_data_small.json'

AIRLINE_SAFETY = r"https://raw.githubusercontent.com/fivethirtyeight/data/master/airline-safety/airline-safety.csv"
MURDEERS_2015 = r"https://raw.githubusercontent.com/fivethirtyeight/data/master/murder_2016/murder_2015_final.csv"
US_BIRTHS = r"https://raw.githubusercontent.com/fivethirtyeight/data/master/births/US_births_2000-2014_SSA.csv"
ZIKA_VIRUS = "https://raw.githubusercontent.com/BuzzFeedNews/zika-data/master/data/raw/paho-who/paho-who-cases-reported-latest.csv"
FIREARM_STATS_US = "https://raw.githubusercontent.com/BuzzFeedNews/nics-firearm-background-checks/master/data/nics-firearm-background-checks.csv"

CARS = 'https://perso.telecom-paristech.fr/eagan/class/igr204/data/cars.csv'
CAUSE_OF_DEATH = 'https://perso.telecom-paristech.fr/eagan/class/igr204/data/CausesOfDeath_France_2001-2008.csv'
BABY_NAMES = 'https://perso.telecom-paristech.fr/eagan/class/igr204/data/nat1900-2017.tsv'
HAPPINESS_STATS = "https://perso.telecom-paristech.fr/eagan/class/igr204/data/happiness.csv"

TABULATOR_DATA = 'http://tabulator.info/sample_data/file_load/data.json'

DATATABLE_ARRAY_URL = 'https://datatables.net/examples/ajax/data/arrays.txt'
DATATABLE_ARRAY_NESTED_URL = 'https://datatables.net/examples/ajax/data/objects_subarrays.txt'
DATATABLE_OBJECTS_URL = 'https://datatables.net/examples/ajax/data/objects.txt'
DATATABLE_OBJECTS_NESTED_URL = 'https://datatables.net/examples/ajax/data/objects_deep.txt'
DATATABLE_ORTHOGONAL_URL = 'https://datatables.net/examples/ajax/data/orthogonal.txt'
DATATABLE_OBJECT_ARRAY_URL = 'https://datatables.net/examples/ajax/data/objects_root_array.txt'

COVID_ECDC_URL = 'https://www.ecdc.europa.eu/sites/default/files/documents/COVID-19-geographic-disbtribution-worldwide-2020-03-21.xlsx'
COVID_TIME_SERIES = 'https://github.com/CSSEGISandData/COVID-19/tree/master/csse_covid_19_data/csse_covid_19_time_series'
COVID_DAILY_DATA = 'https://github.com/CSSEGISandData/COVID-19/tree/master/csse_covid_19_data/csse_covid_19_daily_reports'


""" """

import collections

from epyk.core.js import Imports
from epyk.core.Page import Report

#Imports.installed_packages()


report = Report()
imports = Imports.ImportManager(report=report)

modules = collections.OrderedDict()
print(imports.getModules(modules, 'c3'))

i = imports.cleanImports(['c3'], Imports.JS_IMPORTS)
print(i)

i = imports.cssResolve(['c3'])
print(i)

f = imports.getFiles(['c3'], ['c3'])
print(f['css'][0]['file']['script'])
#

p = imports.getPackage('jqueryui', static_path=r"..\outs")
p = imports.getFullPackage('font-awesome', static_path=r"..\outs")



"""
CSS Style module for the Panels components
"""

from epyk.core.css.styles.classes import CssStyle

"""
.meow:after {
  content: " ";
	border-top: 11px solid #222;
	border-left: 8px solid transparent;
	border-right: 8px solid transparent;
  position: relative;
  top: 111px;
  right: -140px;
}
"""


class CssPanelArrowDown(CssStyle.Style):
  _after = {"content": '" "', "border-left": "50px solid transparent",
         "border-right": "50px solid transparent", "position": "relative",
         "width": 0, "margin": "auto", "display": "block", "bottom": "2px"}

  def customize(self):
    self.after.css({"border-top": "15px solid %s" % self.rptObj.theme.success[1]})

"""
.meow {
  height: 100px;
  width: 300px;
  margin: 30px;
  background: #222222;
}

.meow:after {
  content: " ";
	border-bottom: 20px solid #222;
	border-left: 20px solid transparent;
	border-right: 20px solid transparent;
  position: relative;
  top: -38px; --111px;
  right: -130px;
}
"""


class CssPanelArrowUp(CssStyle.Style):
  _after = {"content": '" "', "border-left": "50px solid transparent",
         "border-right": "50px solid transparent", "position": "relative",
         "width": 0, "display": "block", "bottom": '45px'}

  def customize(self):
    self.after.css({"border-bottom": "15px solid %s" % self.rptObj.theme.success[1]})



"""
Related Pages:

		https://www.w3schools.com/charsets/ref_emoji_smileys.asp
"""

GRINNING_FACE = '&#128512'
GRINNING_FACE_WITH_SMILING_EYES = '&#128513'
FACE_WITH_TEARS_OF_JOY = '&#128514'
SMILING_FACE_WITH_OPEN_MOUTH = '&#128515'
SMILING_FACE_WITH_OPEN_MOUTH_AND_SMILING_EYES = '&#128516'
SMILING_FACE_WITH_OPEN_MOUTH_AND_COLD_SWEAT = '&#128517'
SMILING_FACE_WITH_OPEN_MOUTH_AND_TIGHTLY_CLOSED_EYES = '&#128518'
SMILING_FACE_WITH_HALO = '&#128519'
SMILING_FACE_WITH_HORNS = '&#128520'
WINKING_FACE = '&#128521'
SMILING_FACE_WITH_SMILING_EYES = '&#128522'
FACE_SAVOURING_DELICIOUS_FOOD = '&#128523'
RELIEVED_FACE = '&#128524'
SMILING_FACE_WITH_HEART_SHAPED_EYES = '&#128525'
SMILING_FACE_WITH_SUNGLASSES = '&#128526'
SMIRKING_FACE = '&#128527'
NEUTRAL_FACE = '&#128528'
EXPRESSIONLESS_FACE = '&#128529'
UNAMUSED_FACE = '&#128530'
FACE_WITH_COLD_SWEAT = '&#128531'
PENSIVE_FACE = '&#128532'
CONFUSED_FACE = '&#128533'
CONFOUNDED_FACE = '&#128534'
KISSING_FACE = '&#128535'
FACE_THROWING_A_KISS = '&#128536'
KISSING_FACE_WITH_SMILING_EYES = '&#128537'
KISSING_FACE_WITH_CLOSED_EYES = '&#128538'
FACE_WITH_STUCK_OUT_TONGUE = '&#128539'
FACE_WITH_STUCK_OUT_TONGUE_AND_WINKING_EYE = '&#128540'
FACE_WITH_STUCK_OUT_TONGUE_AND_TIGHTLY_CLOSED_EYES = '&#128541'
DISAPPOINTED_FACE = '&#128542'
WORRIED_FACE = '&#128543'
ANGRY_FACE = '&#128544'
POUTING_FACE = '&#128545'
CRYING_FACE = '&#128546'
PERSEVERING_FACE = '&#128547'
FACE_WITH_LOOK_OF_TRIUMPH = '&#128548'
DISAPPOINTED_BUT_RELIEVED_FACE = '&#128549'
FROWNING_FACE_WITH_OPEN_MOUTH = '&#128550'
ANGUISHED_FACE = '&#128551'
FEARFUL_FACE = '&#128552'
WEARY_FACE = '&#128553'
SLEEPY_FACE = '&#128554'
TIRED_FACE = '&#128555'
GRIMACING_FACE = '&#128556'
LOUDLY_CRYING_FACE = '&#128557'
FACE_WITH_OPEN_MOUTH = '&#128558'
HUSHED_FACE = '&#128559'
FACE_WITH_OPEN_MOUTH_AND_COLD_SWEAT = '&#128560'
FACE_SCREAMING_IN_FEAR = '&#128561'
ASTONISHED_FACE = '&#128562'
FLUSHED_FACE = '&#128563'
SLEEPING_FACE = '&#128564'
DIZZY_FACE = '&#128565'
FACE_WITHOUT_MOUTH = '&#128566'
FACE_WITH_MEDICAL_MASK = '&#128567'



from epyk.core.Page import Report

rptObj = Report()

import sys
# sys.path.append("../../epyk-ui")

from epyk.core.Page import Report
from epyk.tests import data_urls

# import config

# Create a basic report object
rptObj = Report()
rptObj.body.set_background()

data_rest_1 = rptObj.py.requests.csv(data_urls.DATA_EARTHQUAKE, store_location=r"D:\Experiments")
data_rest_2 = rptObj.py.requests.csv(data_urls.AIRLINE_SAFETY, store_location=r"D:\Experiments")

#rptObj.style.
t1 = rptObj.ui.tables.datatables.table(data_rest_2)
# t1.config.fixedHeader.activate()
# t1.config.fixedHeader.headerOffset = 100
# t1.config.colReorder = True
# t1.config.fixedColumns.leftColumns = 1
#t1.config.rowsGroup = [5]
t1.config.responsive.activate()
t1.config.select.activate()
#t1.config.scroller.activate()
# t1.config.buttons = ['copy', 'csv', 'excel', 'pdf']
#t1.config.rowGroup.activate()
# t1.config.scrollX = True
# t1.style.themes.nowrap()

t2 = rptObj.ui.tables.datatables.table(data_rest_2)
#t2.config.autoFill.activate()
t2.config.colReorder.activate()



t3 = rptObj.ui.tables.datatables.table(data_rest_2)
t3.config.searchPanes.cascadePanes = True
t3.config.searchPanes.dtOpts.paging = True
t3.config.searchPanes.panes.header = 'toto'
t3.config.searchPanes.panes.className = 'blabla'

t4 = rptObj.ui.tables.datatables.table(data_rest_2)
t4.config.buttons.enabled = 'excel'
t4.config.buttons.buttons.attr = {'title': "copy"}
# t3.config.searchPanes.columns = [2]
# t3.config.searchPanes.controls = False

#t.style.no_class()
#t.style.themes.bootstrap()
#t.get_column("airline")
#t.config.ajax = "./sources/arrays.txt"

print(rptObj.outs.html_file(name="report_table_datatable"))



from epyk.core.data import DataClass


class OptionConfig(DataClass):

  @property
  def responsive(self):
    """
    https://plot.ly/javascript/configuration-options/
    """
    return self._attrs["responsive"]

  @responsive.setter
  def responsive(self, val):
    self._attrs["responsive"] = val

  @property
  def editable(self):
    """
    https://plot.ly/javascript/configuration-options/
    """
    return self._attrs["editable"]

  @editable.setter
  def editable(self, val):
    self._attrs["editable"] = val

  @property
  def staticPlot(self):
    """
    https://plot.ly/javascript/configuration-options/
    """
    return self._attrs["staticPlot"]

  @staticPlot.setter
  def staticPlot(self, val):
    self._attrs["staticPlot"] = val

  @property
  def scrollZoom(self):
    """
    https://plot.ly/javascript/configuration-options/
    """
    return self._attrs["scrollZoom"]

  @scrollZoom.setter
  def scrollZoom(self, val):
    self._attrs["scrollZoom"] = val



"""

"""

from epyk.core.css.catalogs import Catalog

from epyk.core.css.styles.classes import CssStylesImg


class CatalogImg(Catalog.CatalogGroup):
  def base(self):
    """  """
    return self._set_class(CssStylesImg.CssImgBasic)

  def paragraph(self):
    """  """
    return self._set_class(CssStylesImg.CssImgParagraph)

  def title(self):
    """  """
    return self._set_class(CssStylesImg.CssImgH2)

  def mask(self):
    """
    Description:
    ------------
    CSS Class definition to change a div component to a mask which will cover the parent container
    when the mouse is hover
    """
    return self._set_class(CssStylesImg.CssImgMask)

  def info_link(self):
    """
    Description:
    ------------
    CSS class definition to add shadow and specific colors to a link href component.
    This is used in the image section for actions on the mask or the animated image
    """
    return self._set_class(CssStylesImg.CssImgAInfo)

  def image(self):
    """  """
    return self._set_class(CssStylesImg.CssImg)

  def content(self):
    """  """
    return self._set_class(CssStylesImg.CssContent)

  def view(self):
    """  """
    return self._set_class(CssStylesImg.CssView)

  def carrousel_item(self):
    """  """
    return self._set_class(CssStylesImg.CssCarrouselLi)

  def carrousel_label(self):
    """  """
    return self._set_class(CssStylesImg.CssCarrouselLabel)

  def carrousel_title(self):
    """  """
    return self._set_class(CssStylesImg.CssCarrouselH2)





# Check if pandas is available in the current environment
# if it is the case this module can handle DataFrame directly
try:
  import pandas as pd
  has_pandas = True

except:
  has_pandas = False

from epyk.core import html
from epyk.core import js
from epyk.core.html import graph

from epyk.interfaces.graphs import CompChartsBillboard
from epyk.interfaces.graphs import CompChartsDc
from epyk.interfaces.graphs import CompChartsC3
from epyk.interfaces.graphs import CompChartsChartJs
from epyk.interfaces.graphs import CompChartsPlotly
from epyk.interfaces.graphs import CompChartsNvd3
from epyk.interfaces.graphs import CompChartsD3
from epyk.interfaces.graphs import CompChartsVis
from epyk.interfaces.graphs import CompChartsSvg
from epyk.interfaces.graphs import CompChartsCanvas


class Graphs(object):
  def __init__(self, context):
    self.context = context

  def skillbars(self, records=None, y_column=None, x_axis=None, title=None, width=(100, '%'),
                height=(None, 'px'), htmlCode=None, colUrl=None, colTooltip=None, filters=None, profile=False):
    """
    Description:
    ------------
    Python interface for the HTML Skill bars, simple bars chart done in pure Javascript and CSS

    Example
    records = [
      {"label": 'python', 'value': 12},
      {"label": 'Java', 'value': 5},
      {"label": 'Javascript', 'value': 80}]
    rptObj.ui.charts.skillbars(records, y_column=['value'], x_axis=['label']).css({"width": '100px'})

    Related Pages:

			https://www.w3schools.com/howto/howto_css_skill_bar.asp

    Attributes:
    ----------
    :param records:
    :param y_column:
    :param x_axis:
    :param title:
    :param width:
    :param height:
    :param htmlCode:
    :param colUrl:
    :param colTooltip:
    :param filters:
    :param profile:
    """
    if y_column is None or x_axis is None:
      raise Exception("seriesName and axis must be defined")

    #data = self.context.rptObj.js.data.records(records)
    #jsDataObj = js.AresJs.Js(self.context.rptObj, records, profile=profile).fncs([('percentage', [x_axis], [y_column])])
    html_skillbar = html.HtmlEvent.SkillBar(self.context.rptObj, records, y_column, x_axis, title, width, height, htmlCode,
                                            colUrl, colTooltip, filters, profile)
    self.context.register(html_skillbar)
    return html_skillbar

  def _data(self, data, seriesNames, xAxis, otherDims, dataFncs, xAxisOrder, chartFam, chartType, profile):
    """

    Attributes:
    ----------
    :param data:
    :param seriesNames:
    :param xAxis:
    :param otherDims:
    :param dataFncs:
    :param xAxisOrder:
    :param chartFam:
    :param chartType:
    :param profile:
    """
    if data is None:
      data = []
    if xAxis == '_index' or xAxis is None:
      xAxis = '_index'
      if seriesNames is None:
        seriesNames = list(data.columns)
      data['_index'] = data.index

    if not isinstance(seriesNames, list):
      seriesNames = [seriesNames]
    if dataFncs == False:
      dataFncs = [('all', [xAxis], seriesNames)]
    elif dataFncs is None:
      if otherDims is not None:
        dataFncs = [('sum', [xAxis], seriesNames + list(otherDims), xAxisOrder)]
      else:
        dataFncs = [('sum', [xAxis], seriesNames, xAxisOrder)]
    params = (seriesNames, xAxis) if otherDims is None else tuple([seriesNames, xAxis] + list(otherDims))
    return js.AresJs.Js(self.context.rptObj, data, profile=profile).fncs(dataFncs).output(chartFam, chartType, params)

  def d3script(self, script, aresDf=None, seriesNames=None, xAxis=None, otherDims=None, dataFncs=None, title='',
            globalFilter=None, filterSensitive=True, profile=None, dataSrc=None, xAxisOrder=None, chartOptions=None,
            width=100, widthUnit="%", height=330, heightUnit="px", url=None, htmlCode=None, d3Version=None):
    if aresDf is None:
      aresDf = self.context.rptObj.df([])
    if not hasattr(aresDf, 'htmlCode'):
      if len(aresDf) > 0 and isinstance(aresDf[0], list):
        tmpAresDf = []
        for line in aresDf[1:]:
          tmpAresDf.append(dict(zip(aresDf[0], line)))
        aresDf = tmpAresDf
      aresDf = self.context.rptObj.df(aresDf)
    if xAxis == '_index' or xAxis is None:
      xAxis = '_index'
      if seriesNames is None:
        seriesNames = list(aresDf.columns)
      aresDf['_index'] = aresDf.index
    # if not 'sort_values' in kwargs:
    #   if xAxis is not None and not aresDf.empty:
    #     aresDf.sort_values(by=[xAxis], inplace=True)
    # else:
    #   kwargs['sort_values']["inplace"] = True
    #   aresDf.sort_values(**kwargs['sort_values'])
    #   del kwargs['sort_values']

    if not isinstance(seriesNames, list):
      seriesNames = [seriesNames]
    if dataFncs == False:
      dataFncs = [('all', [xAxis], seriesNames)]
    elif dataFncs is None:
      if otherDims is not None:
        dataFncs = [('sum', [xAxis], seriesNames + list(otherDims), xAxisOrder)]
      else:
        dataFncs = [('sum', [xAxis], seriesNames, xAxisOrder)]
    params = (seriesNames, xAxis) if otherDims is None else tuple([seriesNames, xAxis] + list(otherDims))
    data = js.AresJs.Js(self.context.rptObj, aresDf, profile=profile).fncs(dataFncs).output("D3Bespoke", None, params)
    return self.context.register(getattr(graph, 'GraphD3Bespoke').Chart(self.context.rptObj, script, data,
                                                                        width, widthUnit, height, heightUnit, title, chartOptions, htmlCode, globalFilter, filterSensitive, dataSrc, profile, url, d3Version))

  def sparkline(self, chart_type, data, title=None, options=None, column=None):
    """
    Description:
    ------------
    Display a sparkline component

    Usage::

      rptObj.ui.charts.sparkline("box", [1, 2, 3, 4, 5, 4, 3, 2, 1])
    rptObj.ui.charts.sparkline("bar", [1, 2, 3, 4, 5, 4, 3, 2, 10])

    Related Pages:

			https://omnipotent.net/jquery.sparkline/#s-about

    Attributes:
    ----------
    :param chart_type: The type of chart (bullet, line, bar, tristate, discrete, pie, box)
    :param data: A list of values
    :param options: The chart options
    :param column:

    :rtype: graph.GraphSparklines.Sparklines

    :return: A python Sparkline object
    """
    if column is not None:
      if isinstance(data, list):
        pass

    html_chart = html.graph.GraphSparklines.Sparklines(self.context.rptObj, data, chart_type, title, options)
    self.context.register(html_chart)
    return html_chart


  #  ------------------------------------------------------------------------------------------------------------------
  #
  #
  #  ------------------------------------------------------------------------------------------------------------------
  @property
  def plotly(self):
    """
    Description:
    ------------
    Interface for the Plotly library

    Related Pages:
:return: A Python Plolty object
    """
    return CompChartsPlotly.Plotly(self)

  @property
  def chartJs(self):
    """
    Description:
    ------------
    Interface for the ChartJs library

    Related Pages:
:return: A Python ChartJs object
    """
    return CompChartsChartJs.ChartJs(self)

  @property
  def c3(self):
    """
    Description:
    ------------
    Interface to the Javsacript C3 module

    Related Pages:
:return: A Python C3 object
    """
    return CompChartsC3.C3(self)

  @property
  def billboard(self):
    """
    Description:
    ------------
    Interface to the Javascript Billboard module

    This will propose various charts for data analysis and visualisation based on D£.
    This project has been forked from C3.js

    Related Pages:

			https://naver.github.io/billboard.js/

    :return: A Python Billboard Object
    """
    return CompChartsBillboard.Billboard(self)

  @property
  def vis(self):
    """
    Description:
    ------------
    Interface to the Javascript Vis module

    Related Pages:
:return: A Python Vis object
    """
    return CompChartsVis.Vis(self)

  @property
  def nvd3(self):
    """
    Description:
    ------------
    Interface to the Javascript NVD3 library

    Related Pages:
http://nvd3.org/

    :return: A Python NVD3 object
    """
    return CompChartsNvd3.Nvd3(self)

  @property
  def dc(self):
    """
    Description:
    ------------
    """
    return CompChartsDc.DC(self)

  @property
  def d3(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return CompChartsD3.D3(self)

  @property
  def svg(self):
    """
    Description:
    ------------
    SVG defines vector-based graphics in XML format

    Related Pages:

			https://www.w3schools.com/graphics/svg_intro.asp
    """
    return CompChartsSvg.SVG(self)

  @property
  def canvas(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return CompChartsCanvas.Canvas(self)





from epyk.core.html import Html

from epyk.core.data import DataClass
from epyk.core.data import DataEnum
from epyk.core.data import DataGroup
from epyk.core.data import DataEnumMulti

# The list of CSS classes
from epyk.core.css.styles import GrpClsTable


class Table(Html.Html):
  name, category, callFnc = 'Table', 'Tables', 'table'
  __reqJs = ['ag-grid']

  def __init__(self, report, records, width, height, htmlCode, options, profile):
    data, columns, self.__config = [], [], None
    super(Table, self).__init__(report, [], code=htmlCode, css_attrs={"width": width, "height": height}, profile=profile)
    if records is not None:
      self.config.data = records

  @property
  def config(self):
    if self.__config is None:
      self.__config = TableConfig(self._report)
    return self.__config

  def add_column(self, field, title=None):
    """

    :param field:
    :param title:
    """
    col_def = self.config.columns
    col_def.field = field
    col_def.colId = field
    col_def.headerName = field if title is None else title
    # col_def.filter = True
    return col_def

  @property
  def tableId(self):
    """
    Return the Javascript variable of the chart
    """
    return "%s_obj" % self.htmlId

  def build(self, data=None, options=None, profile=False):
    return 'var %s =  new agGrid.Grid(%s, %s)' % (self.tableId, self.dom.varName, self.config)

  def __str__(self):
    self._report._props.setdefault('js', {}).setdefault("builders", []).append(self.refresh())
    return "<div %s></div>" % (self.get_attrs(pyClassNames=self.style.get_classes()))


class CellEditor(DataGroup):

  def datePicker(self):
    self._attrs["cellEditor"] = "'datePicker'"

  def numericCellEditor(self):
    self._attrs["cellEditor"] = "'numericCellEditor'"

  def agLargeTextCellEditor(self):
    self._attrs["cellEditor"] = "'agLargeTextCellEditor'"

  def agRichSelectCellEditor(self, values, **kwargs):
    """

    https://www.ag-grid.com/javascript-grid-cell-editing/

    :param values:
    :param kwargs:
    """
    self._attrs["cellEditor"] = "'agRichSelectCellEditor'"
    self._attrs["cellEditorParams"] = {'values': values}
    if kwargs:
      self._attrs["cellEditorParams"].update(kwargs)
    return self


class CellRenderer(DataClass):
  def singleClickEditRenderer(self):
    self._attrs["renderer"] = "singleClickEditRenderer"
    return self

  def agGroupCellRenderer(self):
    self._attrs["renderer"] = "agGroupCellRenderer"
    return self


class AggFnc(DataGroup):

  def null(self):
    """
    https://www.ag-grid.com/javascript-grid-column-definitions/
    """
    self._attrs["renderer"] = None

  def sum(self):
    """
    https://www.ag-grid.com/javascript-grid-column-definitions/
    """
    self._attrs["renderer"] = "sum"


class ColType(DataGroup):

  def nonEditableColumn(self):
    """

    https://www.ag-grid.com/javascript-grid-column-definitions/
    """
    self._attrs["type"] = 'nonEditableColumn'

  def dateColumn(self):
    """

    https://www.ag-grid.com/javascript-grid-column-definitions/
    """
    self._attrs["type"] = 'dateColumn'

  def numericColumn(self):
    self._attrs["type"] = 'numericColumn'

  def numberColumn(self):
    self._attrs["type"] = 'numberColumn'

  def medalColumn(self):
    self._attrs["type"] = 'medalColumn'


class ColumnFilter(DataGroup):

  def true(self):
    self._attrs["filter"] = 'true'
    return self

  def false(self):
    self._attrs["filter"] = 'false'
    return self

  def agTextColumnFilter(self, **kwargs):
    """
    A Text Filter for string comparisons.

    https://www.ag-grid.com/javascript-grid-filtering/

    :param kwargs:
    """
    self._attrs["filter"] = "'agTextColumnFilter'"
    if kwargs:
      self._attrs["filterParams"] = kwargs
    return self

  def agDateColumnFilter(self, **kwargs):
    """
    A Date Filter for date comparisons.

    https://www.ag-grid.com/javascript-grid-filter-date/

    :param kwargs:
    """
    self._attrs["filter"] = "'agDateColumnFilter'"
    if kwargs:
      self._attrs["filterParams"] = kwargs
    return self

  def agNumberColumnFilter(self):
    """
    A Number Filter for number comparisons.

    https://www.ag-grid.com/javascript-grid-filtering/
    """
    self._attrs["filter"] = "'agNumberColumnFilter'"
    return self

  def agSetColumnFilter(self, filterName):
    """
    A Set Filter, influenced by how filters work in Microsoft Excel. This is an ag-Grid-Enterprise feature.

    https://www.ag-grid.com/javascript-grid-filter-component/
    :return:
    """
    self._attrs["filter"] = filterName
    return self


class ColOrder(DataGroup):

  def asc(self):
    self._attrs["sort"] = 'asc'
    return self

  def desc(self):
    self._attrs["sort"] = 'desc'
    return self


class Column(DataClass):

  @property
  def children(self):
    """
    Description:
    -----------

    https://www.ag-grid.com/javascript-grid-grouping-headers/
    """
    return self._attrs["children"]

  @children.setter
  def children(self, val):
    self._attrs["children"] = val

  @property
  def colId(self):
    """
    Description:
    -----------

    https://www.ag-grid.com/javascript-grid-grouping-headers/
    """
    return self._attrs["colId"]

  @colId.setter
  def colId(self, val):
    self._attrs["colId"] = val

  @property
  def columnGroupShow(self):
    """
    Description:
    -----------
    values closed, open

    https://www.ag-grid.com/javascript-grid-grouping-headers/
    """
    return self._attrs["columnGroupShow"]

  @columnGroupShow.setter
  def columnGroupShow(self, val):
    self._attrs["columnGroupShow"] = val

  @property
  def editor(self):
    """
    Description:
    -----------
    Cell editing format

    https://www.ag-grid.com/javascript-grid-cell-editing/
    """
    return CellEditor(self, self._attrs)

  @property
  def editable(self):
    """
    Description:
    -----------
    """
    return self._attrs["editable"]

  @editable.setter
  def editable(self, val):
    self._attrs["editable"] = val

  @property
  def enableRowGroup(self):
    """
    Description:
    -----------

    https://www.ag-grid.com/javascript-grid-column-definitions/
    """
    return self._attrs["enableRowGroup"]

  @enableRowGroup.setter
  def enableRowGroup(self, val):
    self._attrs["enableRowGroup"] = val

  @property
  def field(self):
    """
    Description:
    -----------
    """
    return self._attrs["field"]

  @field.setter
  def field(self, val):
    self._attrs["field"] = val

  @property
  def filter(self):
    """
    Description:
    -----------
    Set filtering on a column using the column definition property filter. The property can have one of the following values:

    https://www.ag-grid.com/javascript-grid-filtering/
    """
    return ColumnFilter(self, self._attrs)

  @property
  def hide(self):
    """
    Description:
    -----------
    True if the column is hidden, otherwise false.

    https://www.ag-grid.com/javascript-grid-column-definitions/
    """
    return self._attrs["hide"]

  @hide.setter
  def hide(self, val):
    self._attrs["hide"] = val

  @property
  def headerName(self):
    """
    Description:
    -----------
    """
    return self._attrs["headerName"]

  @headerName.setter
  def headerName(self, val):
    self._attrs["headerName"] = val

  @property
  def lockPinned(self):
    """
    Description:
    -----------
    If you do not want the user to be able to pin using the UI, set the property lockPinned=true.
    This will block the UI in the following way:

    https://www.ag-grid.com/javascript-grid-pinning/
    """
    return self._attrs["lockPinned"]

  @lockPinned.setter
  def lockPinned(self, val):
    self._attrs["lockPinned"] = val

  @property
  def marryChildren(self):
    """
    Description:
    -----------
    Sometimes you want columns of the group to always stick together.
    To achieve this, set the column group property marryChildren=true. The example below demonstrates the following:

    """
    return self._attrs["marryChildren"]

  @marryChildren.setter
  def marryChildren(self, val):
    self._attrs["marryChildren"] = val

  @property
  def sortable(self):
    """
    Description:
    -----------
    """
    return self._attrs["sortable"]

  @sortable.setter
  def sortable(self, val):
    self._attrs["sortable"] = val

  @property
  def filter(self):
    """
    Description:
    -----------
    """
    return self._attrs["filter"]

  @filter.setter
  def filter(self, val):
    self._attrs["filter"] = val

  @property
  def flex(self):
    """
    Description:
    -----------
    It's often required that one or more columns fill the entire available space in the grid. For this scenario,
    it is possible to use the flex config.
    Some columns could be set with a regular width config, while other columns would have a flex config.

    https://www.ag-grid.com/javascript-grid-resizing/
    """
    return self._attrs["flex"]

  @flex.setter
  def flex(self, val):
    self._attrs["flex"] = val

  @property
  def checkboxSelection(self):
    """
    Description:
    -----------
    """
    return self._attrs["checkboxSelection"]

  @checkboxSelection.setter
  def checkboxSelection(self, val):
    self._attrs["checkboxSelection"] = val

  @property
  def suppressMovable(self):
    """
    Description:
    -----------
    The column property suppressMovable changes whether the column can be dragged.

    https://www.ag-grid.com/javascript-grid-column-moving/
    """
    return self._attrs["suppressMovable"]

  @suppressMovable.setter
  def suppressMovable(self, val):
    self._attrs["suppressMovable"] = val

  @property
  def pinned(self):
    """
    Description:
    -----------
    """
    return self._attrs["pinned"]

  @pinned.setter
  def pinned(self, val):
    self._attrs["pinned"] = val

  @property
  def lockPosition(self):
    """
    Description:
    -----------
    The column property lockPosition locks columns to the first position in the grid.

    https://www.ag-grid.com/javascript-grid-column-moving/

    """
    return self._attrs["lockPosition"]

  @lockPosition.setter
  def lockPosition(self, val):
    self._attrs["cellClass"] = 'locked-col'
    self._attrs["lockPosition"] = val

  @property
  def maxWidth(self):
    """
    Description:
    -----------
    """
    return self._attrs["maxWidth"]

  @maxWidth.setter
  def maxWidth(self, val):
    self._attrs["maxWidth"] = val

  @property
  def minWidth(self):
    """
    Description:
    -----------
    """
    return self._attrs["minWidth"]

  @minWidth.setter
  def minWidth(self, val):
    self._attrs["minWidth"] = val

  @property
  def resizable(self):
    """
    Description:
    -----------
    Turn column resizing on for the grid by setting resizable=true for each column.
    To set resizing for each column, set resizable=true on the default column definition.

    https://www.ag-grid.com/javascript-grid-resizing/
    """
    return self._attrs["resizable"]

  @resizable.setter
  def resizable(self, val):
    self._attrs["resizable"] = val

  @property
  def rowGroup(self):
    """
    Description:
    -----------
    """
    return self._attrs["rowGroup"]

  @rowGroup.setter
  def rowGroup(self, val):
    self._attrs["rowGroup"] = val

  @property
  def rowGroupIndex(self):
    """
    Description:
    -----------
    The index of the row group. If the column is not grouped, this field is null. If multiple columns are used to group, this index provides the order of the grouping.

    https://www.ag-grid.com/javascript-grid-column-definitions/
    """
    return self._attrs["rowGroupIndex"]

  @rowGroupIndex.setter
  def rowGroupIndex(self, val):
    self._attrs["rowGroupIndex"] = val

  @property
  def suppressSizeToFit(self):
    """
    Description:
    -----------
    """
    return self._attrs["suppressSizeToFit"]

  @suppressSizeToFit.setter
  def suppressSizeToFit(self, val):
    self._attrs["suppressSizeToFit"] = val

  @property
  def type(self):
    """
    Description:
    -----------

    :return:
    """
    return ColType(self, self._attrs)

  @property
  def enableCellChangeFlash(self):
    """
    Description:
    -----------

    https://www.ag-grid.com/javascript-grid-refresh/
    """
    return self._attrs["enableCellChangeFlash"]

  @enableCellChangeFlash.setter
  def enableCellChangeFlash(self, val):
    self._attrs["enableCellChangeFlash"] = val

  @property
  def suppressCellFlash(self):
    """
    Description:
    -----------

    https://www.ag-grid.com/javascript-grid-refresh/
    """
    return self._attrs["suppressCellFlash"]

  @suppressCellFlash.setter
  def suppressCellFlash(self, val):
    self._attrs["suppressCellFlash"] = val

  @property
  def sortingOrder(self):
    """
    Description:
    -----------
    It is possible to override this behaviour by providing your own sortingOrder on either the gridOptions or the colDef.
    If defined both in colDef and gridOptions, the colDef will get preference, allowing you to defined a common default, and then tailoring per column.

    https://www.ag-grid.com/javascript-grid-sorting/
    """
    return self._attrs["sortingOrder"]

  @sortingOrder.setter
  def sortingOrder(self, val):
    self._attrs["sortingOrder"] = val

  @property
  def sort(self):
    """
    Description:
    -----------
    Cell editing format

    https://www.ag-grid.com/javascript-grid-sorting/
    """
    return ColOrder(self, self._attrs)


class DefaultColDef(DataClass):

  @property
  def filter(self):
    """
    Description:
    -----------
    """
    return self._attrs["filter"]

  @filter.setter
  def filter(self, val):
    self._attrs["filter"] = val

  @property
  def flex(self):
    """
    Description:
    -----------
    It's often required that one or more columns fill the entire available space in the grid. For this scenario,
    it is possible to use the flex config.
    Some columns could be set with a regular width config, while other columns would have a flex config.

    https://www.ag-grid.com/javascript-grid-resizing/
    """
    return self._attrs["flex"]

  @flex.setter
  def flex(self, val):
    self._attrs["flex"] = val

  @property
  def minWidth(self):
    """
    Description:
    -----------
    """
    return self._attrs["minWidth"]

  @minWidth.setter
  def minWidth(self, val):
    self._attrs["minWidth"] = val

  @property
  def editable(self):
    """
    Description:
    -----------
    """
    return self._attrs["editable"]

  @editable.setter
  def editable(self, val):
    self._attrs["editable"] = val

  @property
  def resizable(self):
    """
    Description:
    -----------
    Turn column resizing on for the grid by setting resizable=true for each column.
    To set resizing for each column, set resizable=true on the default column definition.

    https://www.ag-grid.com/javascript-grid-resizing/
    """
    return self._attrs["resizable"]

  @resizable.setter
  def resizable(self, val):
    self._attrs["resizable"] = val

  @property
  def sortable(self):
    """
    Description:
    -----------
    """
    return self._attrs["sortable"]

  @sortable.setter
  def sortable(self, val):
    self._attrs["sortable"] = val


class TableToolPanelsParams(DataClass):

  @property
  def suppressSyncLayoutWithGrid(self):
    """
    Description:
    -----------
    """
    return self._attrs["suppressSyncLayoutWithGrid"]

  @suppressSyncLayoutWithGrid.setter
  def suppressSyncLayoutWithGrid(self, val):
    self._attrs["suppressSyncLayoutWithGrid"] = val


class TableToolPanelsFilters(DataClass):

  @property
  def id(self):
    """
    Description:
    -----------
    """
    return self._attrs["id"]

  @id.setter
  def id(self, val):
    self._attrs["id"] = val

  @property
  def labelDefault(self):
    """
    Description:
    -----------
    """
    return self._attrs["labelDefault"]

  @labelDefault.setter
  def labelDefault(self, val):
    self._attrs["labelDefault"] = val

  @property
  def labelKey(self):
    """
    Description:
    -----------
    """
    return self._attrs["labelKey"]

  @labelKey.setter
  def labelKey(self, val):
    self._attrs["labelKey"] = val

  @property
  def iconKey(self):
    """
    Description:
    -----------
    """
    return self._attrs["iconKey"]

  @iconKey.setter
  def iconKey(self, val):
    self._attrs["iconKey"] = val

  @property
  def toolPanel(self):
    """
    Description:
    -----------
    """
    return self._attrs["toolPanel"]

  @toolPanel.setter
  def toolPanel(self, val):
    self._attrs["toolPanel"] = val

  def toolPanelParams(self):
    """

    :return:
    """
    return self.has_attribute(TableToolPanelsParams)


class TableToolPanels(DataClass):

  def filters(self):
    """

    :return:
    """
    return self.has_attribute(TableToolPanelsFilters)


class TableSideBar(DataClass):

  def toolPanelsColumn(self):
    self._attrs["toolPanels"] = ["columns"]
    return self


class EnumStatusPanelsPanels(DataClass):
  @property
  def statusPanel(self):
    """
    Description:
    -----------
    """
    return self._attrs["statusPanel"]

  @statusPanel.setter
  def statusPanel(self, val):
    self._attrs["statusPanel"] = val

  @property
  def align(self):
    """
    Description:
    -----------
    """
    return self._attrs["align"]

  @align.setter
  def align(self, val):
    self._attrs["align"] = val

  @property
  def key(self):
    """
    Description:
    -----------
    """
    return self._attrs["key"]

  @key.setter
  def key(self, val):
    self._attrs["key"] = val


class TableStatusBar(DataClass):

  def statusPanels(self):
    return self.sub_data_enum('statusPanels', EnumStatusPanelsPanels)


class TableConfig(DataClass):

  @property
  def alignedGrids(self):
    """
    Description:
    -----------
    To have one (the first) grid reflect column changes in another (the second), place the first grid's options in alignedGrids property of the second grids.

    https://www.ag-grid.com/javascript-grid-aligned-grids/
    """
    return self._attrs["alignedGrids"]

  @alignedGrids.setter
  def alignedGrids(self, val):
    self._attrs["alignedGrids"] = val

  @property
  def animateRows(self):
    """
    Description:
    -----------
    """
    return self._attrs["animateRows"]

  @animateRows.setter
  def animateRows(self, val):
    self._attrs["animateRows"] = val

  @property
  def colResizeDefault(self):
    """
    Description:
    -----------
    If you hold 'shift' while dragging the resize handle, the column will take space away from the column adjacent to it.
    This means the total width for all columns will be constant.

    You can also change the default behaviour for resizing.
    Set grid property colResizeDefault='shift' to have shift resizing as default and normal resizing to happen when shift key is pressed.

    https://www.ag-grid.com/javascript-grid-resizing/
    """
    return self._attrs["colResizeDefault"]

  @colResizeDefault.setter
  def colResizeDefault(self, val):
    self._attrs["colResizeDefault"] = val

  @property
  def columns(self):
    """
    Description:
    -----------
    """
    return self.sub_data_enum("columnDefs", Column)

  @property
  def defaultColDef(self):
    """
    Description:
    -----------
    """
    return self.sub_data("defaultColDef", DefaultColDef)

  @property
  def data(self):
    """
    Description:
    -----------
    """
    return self._attrs["rowData"]

  @data.setter
  def data(self, val):
    self._attrs["rowData"] = val

  @property
  def deltaColumnMode(self):
    """
    Description:
    -----------
    """
    return self._attrs["deltaColumnMode"]

  @deltaColumnMode.setter
  def deltaColumnMode(self, val):
    self._attrs["deltaColumnMode"] = val

  @property
  def enablePivot(self):
    """
    Description:
    -----------
    """
    return self._attrs["enablePivot"]

  @enablePivot.setter
  def enablePivot(self, val):
    self._attrs["enablePivot"] = val

  @property
  def enableValue(self):
    """
    Description:
    -----------
    """
    return self._attrs["enableValue"]

  @enableValue.setter
  def enableValue(self, val):
    self._attrs["enableValue"] = val

  @property
  def enterMovesDown(self):
    """
    Description:
    -----------
    Set to true to have Enter key move focus to the cell below if not editing. The default is Enter key starts editing the currently focused cell.

    https://www.ag-grid.com/javascript-grid-cell-editing/
    """
    return self._attrs["enterMovesDown"]

  @enterMovesDown.setter
  def enterMovesDown(self, val):
    self._attrs["enterMovesDown"] = val

  @property
  def enterMovesDownAfterEdit(self):
    """
    Description:
    -----------
    Set to true to have Enter key move focus to the cell below after Enter is pressed while editing. The default is editing will stop and focus will remain on the editing cell.

    https://www.ag-grid.com/javascript-grid-cell-editing/
    """
    return self._attrs["enterMovesDownAfterEdit"]

  @enterMovesDownAfterEdit.setter
  def enterMovesDownAfterEdit(self, val):
    self._attrs["enterMovesDownAfterEdit"] = val

  @property
  def pagination(self):
    """
    Description:
    -----------
    To enable pagination in, set the grid property pagination=true.
    The following simple example shows this, the only difference to this and previous examples is the pagination=true property.

    https://www.ag-grid.com/javascript-grid-pagination/
    """
    return self._attrs["pagination"]

  @pagination.setter
  def pagination(self, val):
    self._attrs["pagination"] = val

  @property
  def paginationPageSize(self):
    """
    Description:
    -----------
    How many rows to load per page. If paginationAutoPageSize is specified, this property is ignored. See Customising Pagination.
    Default: 100

    https://www.ag-grid.com/javascript-grid-pagination/
    """
    return self._attrs.get("paginationPageSize", 100)

  @paginationPageSize.setter
  def paginationPageSize(self, val):
    self._attrs["paginationPageSize"] = val

  @property
  def paginationAutoPageSize(self):
    """
    Description:
    -----------
    If you set paginationAutoPageSize=true the grid will automatically show as many rows in each page as it can fit.
    This is demonstrated below. Note if you resize the display area of the grid, the page size automatically changes.
    To view this, open the example up in a new tab and resize your browser.

    https://www.ag-grid.com/javascript-grid-pagination/
    """
    return self._attrs["paginationAutoPageSize"]

  @paginationAutoPageSize.setter
  def paginationAutoPageSize(self, val):
    self._attrs["paginationAutoPageSize"] = val

  @property
  def groupSelectsChildren(self):
    """
    Description:
    -----------


    https://www.ag-grid.com/javascript-grid-pagination/
    """
    return self._attrs["groupSelectsChildren"]

  @groupSelectsChildren.setter
  def groupSelectsChildren(self, val):
    self._attrs["groupSelectsChildren "] = val

  @property
  def singleClickEdit(self):
    """
    Description:
    -----------
    To change the default so that a single-click starts editing, set the property gridOptions.singleClickEdit = true.
    This is useful when you want a cell to enter edit mode as soon as you click on it, similar to the experience you get when inside Excel.

    https://www.ag-grid.com/javascript-grid-cell-editing/
    """
    return self._attrs["singleClickEdit"]

  @singleClickEdit.setter
  def singleClickEdit(self, val):
    self._attrs["singleClickEdit "] = val

  @property
  def suppressClickEdit(self):
    """
    Description:
    -----------
    The grid configures a cellRenderer with a button to start editing.

    https://www.ag-grid.com/javascript-grid-cell-editing/
    """
    return self._attrs["suppressClickEdit"]

  @suppressClickEdit.setter
  def suppressClickEdit(self, val):
    self._attrs["suppressClickEdit "] = val

  @property
  def suppressPaginationPanel(self):
    """
    Description:
    -----------
    If you set suppressPaginationPanel=true, the grid will not show the standard navigation controls for pagination.
    This is useful is you want to provide your own navigation controls.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-pagination/
    """
    return self._attrs["suppressPaginationPanel"]

  @suppressPaginationPanel.setter
  def suppressPaginationPanel(self, val):
    self._attrs["suppressPaginationPanel "] = val

  @property
  def suppressScrollOnNewData(self):
    """
    Description:
    -----------
    The example also sets property suppressScrollOnNewData=true, which tells the grid to NOT scroll to the top when the page changes.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-pagination/
    """
    return self._attrs["suppressScrollOnNewData"]

  @suppressScrollOnNewData.setter
  def suppressScrollOnNewData(self, val):
    self._attrs["suppressScrollOnNewData "] = val

  @property
  def ensureDomOrder(self):
    """
    Description:
    -----------
    ensures the rows and columns in the DOM always appear in the same order as displayed in the grid.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-cell-editing/
    """
    return self._attrs["ensureDomOrder"]

  @ensureDomOrder.setter
  def ensureDomOrder(self, val):
    self._attrs["ensureDomOrder "] = val

  @property
  def suppressColumnVirtualisation(self):
    """
    Description:
    -----------
    Ensures all columns are rendered, i.e. appears in the DOM.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-accessibility/
    """
    return self._attrs["suppressColumnVirtualisation"]

  @suppressColumnVirtualisation.setter
  def suppressColumnVirtualisation(self, val):
    self._attrs["suppressColumnVirtualisation "] = val

  @property
  def suppressDragLeaveHidesColumns(self):
    """
    Description:
    -----------
    Column animations happen when you move a column. The default is for animations to be turned on.
    It is recommended that you leave the column move animations on unless your target platform (browser and hardware) is to slow to manage the animations.
    To turn OFF column animations, set the grid property suppressColumnMoveAnimation=true.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-column-moving/
    """
    return self._attrs["suppressDragLeaveHidesColumns"]

  @suppressDragLeaveHidesColumns.setter
  def suppressDragLeaveHidesColumns(self, val):
    self._attrs["suppressDragLeaveHidesColumns "] = val

  @property
  def rowMultiSelectWithClick(self):
    """
    Description:
    -----------
    Set to true to allow multiple rows to be selected with clicks.
    For example, if you click to select one row and then click to select another row, the first row will stay selected as well.
    Clicking a selected row in this mode will deselect the row.
    This is useful for touch devices where Ctrl and Shift clicking is not an option.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-selection/
    """
    return self._attrs["rowMultiSelectWithClick"]

  @rowMultiSelectWithClick.setter
  def rowMultiSelectWithClick(self, val):
    self._attrs["rowMultiSelectWithClick "] = val

  @property
  def rowDeselection(self):
    """
    Description:
    -----------
    Set to true to allow rows to be deselected if you hold down Ctrl and click the row.
    By default the grid disallows deselection of rows (i.e. once a row is selected, it remains selected until another row is selected in its place).

    Related Pages:

			https://www.ag-grid.com/javascript-grid-selection/
    """
    return self._attrs["rowDeselection"]

  @rowDeselection.setter
  def rowDeselection(self, val):
    self._attrs["rowDeselection "] = val

  @property
  def rowSelection(self):
    """
    Description:
    -----------
    Type of row selection, set to either 'single' or 'multiple' to enable selection.
    'single' will use single row selection, such that when you select a row, any previously selected row gets unselected.
    'multiple' allows multiple rows to be selected.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-selection/
    """
    return self._attrs["rowSelection"]

  @rowSelection.setter
  def rowSelection(self, val):
    self._attrs["rowSelection "] = val

  @property
  def rowBuffer(self):
    """
    Description:
    -----------
    Sets the number of rows rendered outside of the scrollable viewable area.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-accessibility/
    """
    return self._attrs["rowBuffer"]

  @rowBuffer.setter
  def rowBuffer(self, val):
    self._attrs["rowBuffer "] = val

  @property
  def suppressRowClickSelection(self):
    """
    Description:
    -----------
    If true, rows won't be selected when clicked. Use, for example, when you want checkbox selection, and don't want to also select the row when the row is clicked.

    Related Pages:

			https://www.ag-grid.com/javascript-grid-selection/
    """
    return self._attrs["suppressRowClickSelection"]

  @suppressRowClickSelection.setter
  def suppressRowClickSelection(self, val):
    self._attrs["suppressRowClickSelection "] = val

  @property
  def sortingOrder(self):
    """
    Description:
    -----------
    It is possible to override this behaviour by providing your own sortingOrder on either the gridOptions or the colDef.
    If defined both in colDef and gridOptions, the colDef will get preference, allowing you to defined a common default, and then tailoring per column.

    https://www.ag-grid.com/javascript-grid-sorting/
    """
    return self._attrs["sortingOrder"]

  @sortingOrder.setter
  def sortingOrder(self, val):
    self._attrs["sortingOrder"] = val

  @property
  def sideBar(self):
    """

    :return:
    """
    return self.sub_data("sideBar", TableSideBar)

  def statusBar(self):
    """

    :return:
    """
    return self.sub_data("statusBar", TableStatusBar)



from epyk.core.js.packages import packageImport
from epyk.core.data import DataGroup


class ExtsEditors(DataGroup):

  def text(self, cssMapping, search=True, elementAttributes=None, **kwargs):
    """
    Description:
    -----------
    The input editor allows entering of a single line of plain text

    Related Pages:
http://tabulator.info/docs/4.5/edit#edit-builtin

    Attributes:
    ----------
    :param search: use search type input element with clear button
    :param elementAttributes: set attributes directly on the input element
    """
    self._attrs["editor"] = 'input'
    self._attrs["editorParams"] = {'search': search}
    if elementAttributes is not None:
      self._attrs["editorParams"][elementAttributes] = elementAttributes
    if kwargs:
      self._attrs["editorParams"].update(kwargs)
    self._parent.mutators.text(cssMapping)
    return self

  def number(self, red=None, green=None, search=True, elementAttributes=None, **kwargs):
    """
    Description:
    -----------
    The input editor allows entering of a single line of plain text

    Related Pages:
http://tabulator.info/docs/4.5/edit#edit-builtin

    Attributes:
    ----------
    :param search: use search type input element with clear button
    :param elementAttributes: set attributes directly on the input element
    """
    self._attrs["editor"] = 'input'
    self._attrs["editorParams"] = {'search': search}
    if elementAttributes is not None:
      self._attrs["editorParams"][elementAttributes] = elementAttributes
    if kwargs:
      self._attrs["editorParams"].update(kwargs)
    self._parent.mutators.number(red, green)
    return self

  @packageImport('editors-inputs')
  def input(self):
    self._attrs["editor"] = 'inputPlus'
    return self

  @packageImport('editors-inputs')
  def input_excel(self):
    self._attrs["editor"] = 'inputExcel'
    return self

  @packageImport('editors-dates')
  def date(self):
    self._attrs["editor"] = 'datePlus'
    return self

  @packageImport('editors-dates')
  def date(self):
    self._attrs["editor"] = 'datePlus'
    return self

  @packageImport('editors-selects')
  def select(self):
    self._attrs["editor"] = 'datePlus'
    return self

  @packageImport('editors-selects')
  def select_rule(self):
    self._attrs["editor"] = 'selectConditiions'
    return self

  @packageImport('editors-selects')
  def select_multi_rules(self):
    self._attrs["editor"] = 'selectMultiConditions'
    return self

  def custom(self, formatter, formatterParams, moduleAlias):
    """

    :param formatter:
    :param formatterParams:
    :param moduleAlias:
    """
    self._report.jsImports.add(moduleAlias)
    self._attrs["editor"] = formatter
    self._attrs['editorParams'] = formatterParams
    return self



"""

"""

from epyk.core.css.styles import GrpCls
from epyk.core.css import Classes
from epyk.core.css.styles.attributes import AttrClsChart


class ClassBSpartlines(GrpCls.ClassHtml):
  @property
  def css(self):
    """
    Property to the underlying CSS definition to be added to the style HTML tag of a component
    :rtype: AttrClsChart.AttrSkarkline
    """
    if self._css_struct is None:
      self._css_struct = AttrClsChart.AttrSkarkline(self.htmlObj)
    return self._css_struct

  @property
  def css_class(self):
    """
    The internal class used to put a custom Style to this object.
    Only 1 CSS class can be added to an HTML object
    :rtype: Classes.CatalogDiv.CatalogDiv
    """
    if self._css_class is None:
      self._css_class = Classes.CatalogDiv.CatalogDiv(self.htmlObj._report, self.classList['main']).margin_vertical()
    return self._css_class




from epyk.core import html
from epyk.core.html import Defaults_html
from epyk.core.css import Defaults_css


class Texts(object):

  def __init__(self, context):
    self.context = context

  def text(self, text="", color=None, align='left', width=('auto', ""), height=(None, "px"),
           htmlCode=None, tooltip=None, options=None, helper=None, profile=None):
    """
    Description:
    ------------
    Add the HTML text component to the page

    Usage::

      rptObj.ui.text("this is a test")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Text`

    Related Pages:

			https://www.w3schools.com/tags/tag_font.asp

    Attributes:
    ----------
    :param text: The string value to be displayed in the component
    :param color: Optional. The color of the text
    :param align: Optional. The position of the icon in the line (left, right, center)
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param tooltip: Optional. A string with the value of the tooltip
    :param options: Optional. The component options
    :param helper:
    :param profile: Optional. A flag to set the component performance storage

    :return: The text HTML object
    """
    if not isinstance(width, tuple):
      width = (width, 'px')
    text_comp = html.HtmlText.Text(self.context.rptObj, text, color, align, width, height, htmlCode, tooltip,
                                   options or {}, helper, profile)
    self.context.register(text_comp)
    return text_comp

  def label(self, text="", color=None, align='center', width=(100, "px"), height=('auto', ""), htmlCode=None,
            tooltip='', profile=None, options=None):
    """
    Description:
    ------------
    The <label> tag defines a label for a <button>, <input>, <meter>, <output>, <progress>, <select>, or <textarea> element...

    The for attribute of the <label> tag should be equal to the id attribute of the related element to bind them together.

    Usage::

      rptObj.ui.texts.label("Test")
      rptObj.ui.texts.label("this is a test", color="red")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Label`

    Related Pages:

			https://www.w3schools.com/tags/tag_label.asp

    Attributes:
    ----------
    :param text: Optional. The string value to be displayed in the component
    :param color: Optional. The color of the text
    :param align: Optional. The position of the icon in the line (left, right, center)
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param tooltip: Optional. A string with the value of the tooltip
    :param profile: Optional. A flag to set the component performance storage
    """
    dflt_options = {"markdown": True}
    if options is not None:
      dflt_options.update(options)
    html_label = html.HtmlText.Label(self.context.rptObj, text, color, align, width, height, htmlCode, tooltip,
                                     profile, dflt_options)
    self.context.register(html_label)
    return html_label

  def span(self, text="", color=None, align='center', width=None, height=None, htmlCode=None,
           tooltip=None, options=None, profile=None):
    """
    Description:
    ------------
    The <span> tag is used to group inline-elements in a document.

    The <span> tag provides no visual change by itself.

    The <span> tag provides a way to add a hook to a part of a text or a part of a document.

    Usage::

      rptObj.ui.texts.span("Test")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Span`

    Related Pages:

			https://www.w3schools.com/tags/tag_span.asp

    Attributes:
    ----------
    :param text: Optional. The string value to be displayed in the component
    :param color: Optional. The color of the text
    :param align: Optional. The position of the icon in the line (left, right, center)
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param tooltip: Optional. A string with the value of the tooltip
    :param profile: Optional. A flag to set the component performance storage
    """
    if width is None:
      width = (Defaults_html.TEXTS_SPAN_WIDTH, 'px')
    if height is None:
      height = (Defaults_html.LINE_HEIGHT, 'px')
    html_label = html.HtmlText.Span(self.context.rptObj, text, color, align, width, height, htmlCode, tooltip, options, profile)
    self.context.register(html_label)
    return html_label

  def highlights(self, text=None, title="", icon=None, type="danger", color=None, width=(None, "%"),
                 height=(None, "px"), htmlCode=None, helper=None, profile=None):
    """
    Description:
    ------------
    Provide contextual feedback messages for typical user actions with the handful of available and flexible alert messages.

    Usage::

      rptObj.ui.texts.highlights("Test content", title="Test", icon="fab fa-angellist")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Highlights`

    Related Pages:

			https://getbootstrap.com/docs/4.3/components/alerts/

    Attributes:
    ----------
    :param text: Optional. The string value to be displayed in the component
    :param title:
    :param icon:
    :param type: Optional, The type of the warning. Can be (primary, secondary, success, danger, warning, info, light,
                 dark). Default danger
    :param color:
    :param width:
    :param height:
    :param htmlCode:
    :param helper:
    :param profile:
    """
    html_light = html.HtmlText.Highlights(self.context.rptObj, text, title, icon, type, color, width,
                                          height, htmlCode, helper, profile)
    self.context.register(html_light)
    return html_light

  def formula(self, text=None, width=(100, "%"), color=None, helper=None, profile=None):
    """
    Description:
    ------------
    Interface to the mathjs Formulas object

    Usage::

      rptObj.ui.texts.formula("$$x = {-b \pm \sqrt{b^2-4ac} \over 2a}.$$", helper="This is a formula")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlTextComp.Formula`

    Related Pages:

      https://mathjs.org/docs/index.html


    Attributes:
    ----------
    :param text: Optional. The string value to be displayed in the component
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param color: Optional. The color of the text
    :param helper:
    :param profile: Optional. A flag to set the component performance storage
    """
    html_formula = html.HtmlTextComp.Formula(self.context.rptObj, text, width, color, helper, profile)
    self.context.register(html_formula)
    return html_formula

  def code(self, text="", language='python', color=None, width=(90, '%'), height=(200, 'px'), htmlCode=None,
           options=None, helper=None, profile=None):
    """
    Description:
    ------------
    Python Wrapper to the Bootstrap CODE Tag

    Usage::

      rptObj.ui.texts.code("This is a code")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlTextEditor.Code`

    Related Pages:

			https://v4-alpha.getbootstrap.com/content/code/

    Attributes:
    ----------
    :param text:
    :param color:
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param helper:
    :param profile:
    """
    dflt_options = {"lineNumbers": True, 'mode': language, 'matchBrackets': True, 'styleActiveLine': True, 'autoRefresh': True}
    if options is not None:
      dflt_options.update(options)
    html_code = html.HtmlTextEditor.Code(self.context.rptObj, text, color, width, height, htmlCode, dflt_options, helper, profile)
    self.context.register(html_code)
    return html_code

  def paragraph(self, text="", color=None, background_color=None, border=False, width=(100, "%"),
                height=(None, 'px'), htmlCode=None, encoding="UTF-8", dataSrc=None, helper=None, options=None, profile=None):
    """
    Description:
    ------------
    Python Wrapper to the HTML P Tag

    Usage::

      rptObj.ui.texts.paragraph("This is a paragraph", helper="Paragraph helper")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Paragraph`

    Related Pages:

			https://www.w3schools.com/html/html_styles.asp

    Attributes:
    ----------
    :param text:
    :param color:
    :param background_color:
    :param border:
    :param width:
    :param height:
    :param htmlCode:
    :param encoding:
    :param dataSrc:
    :param profile:
    """
    dflt_options = {"reset": True, 'markdown': False, "classes": []}
    if options is not None:
      dflt_options.update(options)
    html_paragraph = html.HtmlText.Paragraph(self.context.rptObj, text, color, background_color, border, width, height,
                                             htmlCode, encoding, dataSrc, helper, dflt_options, profile)
    self.context.register(html_paragraph)
    return html_paragraph

  def preformat(self, text=None, color=None, width=(90, '%'), height=(None, 'px'), htmlCode=None, dataSrc=None,
                options=None, helper=None, profile=None):
    """
    Description:
    ------------
    Preformatted text:
    The <pre> tag defines preformatted text.
    Text in a <pre> element is displayed in a fixed-width font (usually Courier), and it preserves both spaces and line breaks.

    Usage::

      rptObj.ui.texts.preformat("This is a pre formatted text")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Pre`

    Related Pages:

			https://www.w3schools.com/html/html_styles.asp
      https://www.w3schools.com/tags/tag_pre.asp

    Attributes:
    ----------
    :param text:
    :param color:
    :param width:
    :param height:
    :param htmlCode:
    :param dataSrc:
    :param options:
    :param helper:
    :param profile:
    """
    dflt_options = {"reset": True, 'markdown': False}
    if options is not None:
      dflt_options.update(options)
    html_pre = html.HtmlText.Pre(self.context.rptObj, text, color, width, height, htmlCode, dataSrc, dflt_options, helper, profile)
    self.context.register(html_pre)
    return html_pre

  def blockquote(self, text=None, author=None, color=None, width=(None, '%'), height=(None, 'px'),
                 htmlCode=None, helper=None, options=None, profile=None):
    """
    Description:
    ------------
    The <blockquote> tag specifies a section that is quoted from another source.
    Browsers usually indent <blockquote> elements.

    Usage::

      rptObj.ui.texts.blockquote("This is a code")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.BlockQuote`

    Related Pages:

			https://v4-alpha.getbootstrap.com/content/typography/
      https://www.w3schools.com/TAGS/tag_blockquote.asp

    Attributes:
    ----------
    :param text:
    :param author:
    :param color:
    :param width:
    :param height:
    :param htmlCode:
    :param helper:
    :param profile:
    """
    html_blockquote = html.HtmlText.BlockQuote(self.context.rptObj, text, author, color, width, height, htmlCode, helper, options, profile)
    self.context.register(html_blockquote)
    return html_blockquote

  def up_down(self, rec=None, color=None, label=None, options=None, helper=None, profile=None):
    """
    Description:
    ------------
    Up and down Text component

    Usage::

      rptObj.ui.texts.up_down({'previous': 240885, 'value': 240985})

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.UpDown`

    Related Pages:

			https://fontawesome.com/

    Attributes:
    ----------
    :param rec:
    :param color:
    :param label:
    :param options:
    :param helper:
    :param profile:
    """
    dflt_options = {"digits": 0, 'thousand_sep': ",", 'decimal_sep': ".", 'font_size': Defaults_css.font(),
                    'red': self.context.rptObj.theme.danger[1], 'green': self.context.rptObj.theme.success[1],
                    'orange': self.context.rptObj.theme.warning[1]}
    if options is not None:
      dflt_options.update(options)
    html_up_down = html.HtmlTextComp.UpDown(self.context.rptObj, rec, color, label, dflt_options, helper, profile)
    self.context.register(html_up_down)
    return html_up_down

  def number(self, number=0, title=None, label=None, icon=None, color=None, tooltip='', htmlCode=None,
             options=None, helper=None, width=(150, 'px'), profile=None):
    """
    Description:
    ------------

    Usage::

      rptObj.ui.texts.number(289839898, label="test", helper="Ok", icon="fas fa-align-center")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Numeric`

    Related Pages:

      http://openexchangerates.github.io/accounting.js/

    Attributes:
    ----------
    :param number: Optional. The value to be displayed to the component. Default now
    :param title:
    :param label: Optional. The text of label to be added to the component
    :param icon: Optional. A string with the value of the icon to display from font-awesome
    :param color:
    :param tooltip:
    :param htmlCode:
    :param options:
    :param helper:
    :param profile:
    """
    dflt_options = {"digits": 0, "thousand_sep": ',', "decimal_sep": '.'}
    if options is not None:
      dflt_options.update(options)
    html_number = html.HtmlText.Numeric(self.context.rptObj, number, title, label, icon, color, tooltip, htmlCode,
                                        dflt_options, helper, width, profile)
    self.context.register(html_number)
    return html_number

  def title(self, text=None, level=None, name=None, contents=None, color=None, picture=None, icon=None,
            marginTop=5, htmlCode=None, width=("auto", ""), height=(None, "px"), align=None, options=None, profile=None):
    """
    Description:
    ------------
    Add a title

    Usage::

      rptObj.ui.title("Test")
      rptObj.ui.title("Test", level=2)

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Title`

    Related Pages:

			https://www.w3schools.com/tags/tag_hn.asp

    Attributes:
    ----------
    :param text:
    :param level:
    :param name:
    :param contents:
    :param color:
    :param picture:
    :param icon:
    :param marginTop:
    :param htmlCode:
    :param width:
    :param height:
    :param align:
    :param options:
    :param profile:
    """
    dflt_options = {"reset": True, 'markdown': False}
    if options is not None:
      dflt_options.update(options)
    html_title = html.HtmlText.Title(self.context.rptObj, text, level, name, contents, color, picture, icon,
                                     marginTop, htmlCode, width, height, align, dflt_options, profile)
    self.context.register(html_title)
    return html_title

  def fieldset(self, legend="", width=(100, "%"), height=(None, "px"), helper=None, options=None, profile=None):
    """
    Description:
    ------------
    The <fieldset> tag is used to group related elements in a form.
    The <fieldset> tag draws a box around the related elements.

    Usage::

      rptObj.ui.texts.fieldset("legend")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlText.Fieldset`

    Related Pages:

			https://www.w3schools.com/tags/tag_legend.asp
      https://www.w3schools.com/tags/tag_fieldset.asp

    Attributes:
    ----------
    :param legend:
    :param width:
    :param height:
    :param options:
    :param profile:
    """
    html_fieldset = html.HtmlText.Fieldset(self.context.rptObj, legend, width=width, height=height, helper=helper,
                                           options=options, profile=profile)
    self.context.register(html_fieldset)
    return html_fieldset



"""
"""
import sys
import argparse
import pkg_resources
import shutil
import os



def main():
  """"""
  parser_map = {'env':           (create_env_parser,         '''Create new environment'''),
                'deploy':        (create_deploy_parser,      '''Deploy latest changes'''),
                'db':           (create_db_parser,          '''Performs operation on local DB (Sqlite)'''),
                'get_packages': (create_import_pkg_parser,  '''Downloads Javascript and CSS packages to allow offline development'''),
                'version':      (create_version_parser,     '''Informs on current package version'''),
                'notebooks':    (create_notebook_parser,    '''Donwloads or Upload Jupyter notebooks online''')
                }
  arg_parser = argparse.ArgumentParser(prog='epyk')
  subparser = arg_parser.add_subparsers(title='Commands', dest='command')
  subparser.required = True
  for func, parser_init in parser_map.items():
    new_parser = subparser.add_parser(func, help=parser_init[1])
    parser_init[0](new_parser)
  args = arg_parser.parse_args(sys.argv[1:])
  return args.func(args)


def create_env_parser(subparser):
  """"""
  subparser.set_defaults(func=env)
  subparser.add_argument('-p', '--path', required=True, help='''The path where the new environment will be created: -p /foo/bar''')
  subparser.add_argument('-n', '--name', default='NewEnv', help='''The name of the new environment: -n MyEnv''')
  subparser.add_argument('--from', help='''Path or URL of the environment to be copied from: --from /foo/bar/oldEnv or --from http://repo/env1/getEnv''')
  subparser.add_argument('--only', nargs='+', help='''If the from option is specified this will allow only certain folder/files to be copied: --only script1.py folder/scripts*.py''')

def create_deploy_parser(subparser):
  """"""
  subparser.set_defaults(func=deploy)
  subparser.add_argument('-p', '--path', nargs='+', required=True, help='''The path where the environment you want to deploy is: -p /foo/bar/myEnv''')

def create_db_parser(subparser):
  """"""
  subparser.set_defaults(func=db)
  subparser.add_argument('-p', '--path', required=True, help='''The path where the new environment will be created: -p /foo/bar''')
  subparser.add_argument('-m', '--migrate', help='''The path where the new environment will be created: -p /foo/bar''')

def create_import_pkg_parser(subparser):
  """"""
  subparser.set_defaults(func=get_packages)
  subparser.add_argument('-p', '--path', required=True, help='''The path of the project where the package will be download: -p /foo/bar''')
  subparser.add_argument('-o', '--only', nargs='+', default=['all'], help='''The name of the package to be downloaded (by default we will download all packages): -n d3.min.js''')
  subparser.add_argument('-x', '--exclude', help='''packages to be excluded: -x d3.min.js''')


def create_version_parser(subparser):
  """"""
  subparser.set_defaults(func=version)

def create_notebook_parser(subparser):
  """"""
  subparser.set_defaults(func=notebooks)

def env(args):
  """
   Creates a new epyk environment on disk
  """
  folder_lst = ['data', 'model', os.path.join('model', 'sqlite'), os.path.join('model', 'sqlite', 'local'), 'static']
  env_path = os.path.join(args.path, args.name)
  if os.path.exists(env_path):
    raise argparse.ArgumentTypeError('An environment with this name already exists at this location: {}'.format(env_path))

  os.makedirs(env_path)
  open(os.path.join(env_path, '__init__.py'), 'w').close()
  for folder in folder_lst:
    cur_folder = os.path.join(env_path, folder)
    os.makedirs(cur_folder)
    if 'model' in cur_folder:
      open(os.path.join(cur_folder, '__init__.py'), 'w').close()
  print('Environment created!')

def deploy(args):
  """

  """
  pass

def db(args):
  """"""
  pass

def get_packages(args):
  """"""
  try:
    from epyk.core.js.Imports import ImportManager
  except (ImportError, ModuleNotFoundError) as e:
    sys.path.append(os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '..', '..', '..')))
    from epyk.core.js.Imports import ImportManager
  static_path = args.path
  try:
    shutil.copytree(os.path.join(os.path.abspath(os.path.dirname(__file__)), '..', '..', 'static'),
                      os.path.join(static_path, 'static'))
  except FileExistsError:
    pass

  if 'all' in args.only:
    exclude = args.exclude if args.exclude else None
    ImportManager().getPackages(static_path=static_path, exclude=exclude)
  else:
    for package in args.only:
      ImportManager().getPackage(package, reload=True, static_path=static_path)

def version(args):
  """
  Returns the package version for Epyk
  """
  print('Epyk Version: %s' % pkg_resources.get_distribution('epyk').version)

def notebooks(args):
  """Allows you to download or upload notebooks"""
  pass

if __name__ == '__main__':
  main()




from epyk.core.css.styles import GrpCls
from epyk.core.css import Classes
from epyk.core.css.styles.attributes import AttrInput


class ClassInput(GrpCls.ClassHtml):
  @property
  def css(self):
    """
    Property to the underlying CSS definition to be added to the style HTML tag of a component
    :rtype: AttrInput.AttrInput
    """
    if self._css_struct is None:
      self._css_struct = AttrInput.AttrInput(self.htmlObj)
    return self._css_struct

  @property
  def css_class(self):
    """
    The internal class used to put a custom Style to this object.
    Only 1 CSS class can be added to an HTML object
    :rtype: Classes.CatalogInput.CatalogInput
    """
    if self._css_class is None:
      self._css_class = Classes.CatalogInput.CatalogInput(self.htmlObj._report, self.classList['main']).basic()
    return self._css_class


class ClassInputDate(ClassInput):

  def __init__(self, htmlObj):
    super(ClassInputDate, self).__init__(htmlObj)
    self._css_class_dt, self._css_class_dt_ui, self._css_time_picker_header = None, None, None
    self.classList['main'].add(self.cls_datepicker)
    self.classList['main'].add(self.cls_datepicker_ui)
    self.classList['other'].add(self.cls_datepicker_header)

  @property
  def cls_datepicker(self):
    """

    :rtype: Classes.CatalogInput.CatalogDate
    """
    if self._css_class_dt is None:
      self._css_class_dt = Classes.CatalogInput.CatalogDate(self.htmlObj._report, self.classList['main']).datepicker()
    return self._css_class_dt

  @property
  def cls_datepicker_ui(self):
    """

    :rtype: Classes.CatalogInput.CatalogDate
    """
    if self._css_class_dt_ui is None:
      self._css_class_dt_ui = Classes.CatalogInput.CatalogDate(self.htmlObj._report, self.classList['main']).datepicker_ui()
    return self._css_class_dt_ui

  @property
  def cls_datepicker_header(self):
    """

    :rtype: Classes.CatalogInput.CatalogDate
    """
    if self._css_time_picker_header is None:
      self._css_time_picker_header = Classes.CatalogInput.CatalogDate(self.htmlObj._report, self.classList['other']).datepicker_header()
    return self._css_time_picker_header


class ClassInputTime(ClassInput):

  def __init__(self, htmlObj):
    super(ClassInputTime, self).__init__(htmlObj)
    self._css_class_tm, self._css_class_tm_it = None, None
    self.classList['main'].add(self.cls_timepicker)
    self.classList['main'].add(self.cls_timepicker_items)

  @property
  def cls_timepicker(self):
    """

    :rtype: Classes.CatalogInput.CatalogDate
    """
    if self._css_class_tm is None:
      self._css_class_tm = Classes.CatalogInput.CatalogDate(self.htmlObj._report, self.classList['main']).time_picker()
    return self._css_class_tm

  @property
  def cls_timepicker_items(self):
    """

    :rtype: Classes.CatalogInput.CatalogDate
    """
    if self._css_class_tm_it is None:
      self._css_class_tm_it = Classes.CatalogInput.CatalogDate(self.htmlObj._report, self.classList['main']).time_picker_items()
    return self._css_class_tm_it


class ClassInputRange(GrpCls.ClassHtml):

  def __init__(self, htmlObj):
    super(ClassInputRange, self).__init__(htmlObj)
    self._css_class_rg, self._css_class_rg_tb = None, None
    self.classList['main'].add(self.cls_range)
    self.classList['main'].add(self.cls_range_thumb)

  @property
  def cls_range(self):
    """

    :rtype: Classes.CatalogInput.CatalogInput
    """
    if self._css_class_rg is None:
      self._css_class_rg = Classes.CatalogInput.CatalogInput(self.htmlObj._report, self.classList['main']).range()
    return self._css_class_rg

  @property
  def cls_range_thumb(self):
    """

    :rtype:Classes.CatalogInput.CatalogInput
    """
    if self._css_class_rg_tb is None:
      self._css_class_rg_tb = Classes.CatalogInput.CatalogInput(self.htmlObj._report, self.classList['main']).range_thumb()
    return self._css_class_rg_tb





class OptionsButton(object):
  def __init__(self, src, options):
    self.src = src
    self._multiple, self.src.attr['name'] = options.get('multiple', False), options.get('group', None)

  @property
  def multiple(self):
    """
    Property to define if multiple buttons can be selected at the same time
    Default value is false
    """
    return self._multiple

  @multiple.setter
  def multiple(self, bool):
    self._multiple = bool

  @property
  def group(self):
    """
    Property to set the group name of a button
    """
    return self.src.attr['name']

  @group.setter
  def group(self, val):
    self.src.attr['name'] = val


class OptionsBadge(object):
  def __init__(self, src, options):
    self.src = src
    self._badge_prop = options.get('badge_position', "left")
    if self._badge_prop == 'left':
      self._badge_css = {"position": 'relative'}
    else:
      self._badge_css = {"position": 'relative', "top": "-4px", "right": "11px"}
    self._badge_css.update(options.get('badge_css', {}))

  @property
  def badge_css(self):
    """
    """
    return self._badge_css

  @badge_css.setter
  def badge_css(self, css):
    self.src.link.css(css)
    self._badge_css = css

  @property
  def badge_position(self):
    """

    :return:
    """
    return self._badge_prop

  @badge_position.setter
  def badge_position(self, position):
    """

    :param position:
    :return:
    """
    self._badge_prop = position



"""
CSS Style module for the Files Drop components
"""

from epyk.core.css.styles.classes import CssStyle


class CssDropFile(CssStyle.Style):
  _attrs = {'text-align': 'center', 'padding': '5px', 'margin': '5px 0 10px 0'}

  def customize(self):
    self.css({'border': '1px dashed %s' % self.rptObj.theme.colors[1], 'color': self.rptObj.theme.colors[1]})



from epyk.core.css.styles.classes import CssStyle


class CssImgBasic(CssStyle.Style):
  _attrs = {'display': 'inline-block', 'position': 'relative', 'height': 'auto', 'max-height': '100%',
            'max-width': '100%'}
  _selectors = {'child': 'img'}


class CssImgParagraph(CssStyle.Style):
  _attrs = {'transform': 'scale(1.1)', 'font-style': 'italic', 'padding': '10px', 'position': 'relative'}
  _hover = {'transition': 'all 0.2s linear'}
  _selectors = {'child': 'p'}

  def customize(self):
    self.css({'color': self.rptObj.theme.colors[0]})


class CssImgH2(CssStyle.Style):
  _attrs = {'opacity': 0, 'transition': 'all 0.2s ease-in-out', 'text-transform': 'uppercase', 'text-align': 'center',
            'position': 'relative', 'padding': '10px', 'margin': '20px 0 0 0'}
  _hover = {'opacity': 1, 'transform': 'translateY(0px)'}
  _selectors = {'child': 'h2'}

  def customize(self):
    self.css({'color': self.rptObj.theme.colors[0]})


class CssImgMask(CssStyle.Style):
  _attrs = {'opacity': 0, 'transition': 'all 0.4s ease-in-out', 'height': 'inherit', 'left': 1,
            'position': 'absolute'}
  _hover = {'opacity': 0.8}

  def customize(self):
    self.css({'background-color': self.rptObj.theme.success[0], 'border': '1px solid %s' % self.rptObj.theme.success[1]})


class CssImgAInfo(CssStyle.Style):
  _attrs = {'opacity': 1, 'transition': 'all 0.2s ease-in-out', 'display': 'block', 'text-decoration': 'none',
            'padding': '7px 14px', 'text-transform': 'uppercase', 'box-shadow': '0 0 1px #000', "margin": '10px auto',
            'text-align': "center"}
  _hover = {'transform': 'translateY(0px)', 'box-shadow': '0 0 5px #000', 'transition-delay': '0.2s'}

  def customize(self):
    self.css({'background-color': self.rptObj.theme.colors[0], 'color': self.rptObj.theme.colors[-1]})


class CssImg(CssStyle.Style):
  _attrs = {'transition': 'all 0.2s linear', 'display': 'block', 'height': 'auto', 'margin': 'auto', 'max-height': '100%',
            'max-width': '100%', 'position': 'relative'}
  _hover = {'transform': 'scale(1.1)'}
  _selectors = {'child': 'img'}


class CssContent(CssStyle.Style):
  _attrs = {'width': '100%', 'height': '70%', 'position': 'absolute', 'overflow': 'hidden', 'top': '20%', 'left': 0}
  classname = 'content'


class CssView(CssStyle.Style):
  _attrs = {'height': '70%', 'margin': '10px', 'float': 'left', 'overflow': 'hidden', 'position': 'relative',
            'text-align': 'center', 'cursor': 'default'}

  def customize(self):
    self.css({'border': '1px solid %s' % self.rptObj.theme.greys[5]})
    self.hover.css({'border': "1px solid %s" % self.rptObj.theme.success[1]})


class CssCarrouselLi(CssStyle.Style):
  _attrs = {'list-style': 'none', 'display': 'none', 'padding': 0, 'margin': 0}
  _selectors = {'child': 'li'}


class CssCarrouselLabel(CssStyle.Style):
  _attrs = {'background': 'black', 'padding': '10px', 'border-radius': '50%', 'display': 'inline-block',
            'text-align': 'center'}
  _selectors = {'child': 'label'}


class CssCarrouselH2(CssStyle.Style):
  _attrs = {'position': 'absolute', 'top': '10px', 'padding': '10px'}
  _selectors = {'child': 'h2'}




from epyk.core.js import JsUtils
from epyk.core.js.packages import JsPackage
from epyk.core.js.primitives import JsObjects


class CM(JsPackage):
  lib_alias = {"js": 'codemirror', 'css': 'codemirror'}
  lib_set_var = False

  def __init__(self, htmlObj, varName=None, setVar=False, report=None):
    super(CM, self).__init__(htmlObj, varName=varName, selector=htmlObj.editorId, data=None, setVar=setVar, parent=None)
    self._src, self._report = htmlObj, report

  def setSize(self, width=None, height=None):
    """
    Description:
    -----------
    Programmatically set the size of the editor (overriding the applicable CSS rules).
    width and height can be either numbers (interpreted as pixels) or CSS units ("100%", for example).
    You can pass null for either of them to indicate that that dimension should not be changed.

    Related Pages:

			https://codemirror.net/doc/manual.html#option_viewportMargin

    Attributes:
    ----------
    :param width: String or integer. The width of the component
    :param height: String or integer. The height of the component

    :return: The Javascript string fragment
    """
    width = JsUtils.jsConvertData(width, None)
    height = JsUtils.jsConvertData(height, None)
    return self.fnc_closure("setSize(%s, %s)" % (width, height))

  def scrollTo(self, x=None, y=None):
    """
    Description:
    -----------
    Scroll the editor to a given (pixel) position. Both arguments may be left as null or undefined to have no effect.

    Related Pages:

			https://codemirror.net/doc/manual.html#option_viewportMargin

    Attributes:
    ----------
    :param x: Integer. The x position of the scrollbar
    :param y: Integer. The y position of the scrollbar

    :return: The Javascript string fragment
    """
    return self.fnc_closure("scrollTo(%s, %s)" % (x, y))

  def hasFocus(self):
    """
    Description:
    -----------
    Tells you whether the editor currently has focus.

    Related Pages:

			https://codemirror.net/doc/manual.html#option_viewportMargin

    :return: A Boolean
    """
    return JsObjects.JsBoolean.JsBoolean.get("%s.hasFocus()" % self.varId)

  def setOption(self, option, value):
    """
    Description:
    -----------
    Change the configuration of the editor.
    option should the name of an option, and value should be a valid value for that option.

    Related Pages:

			https://codemirror.net/doc/manual.html#option_viewportMargin

    Attributes:
    ----------
    :param option: String. The editor option name
    :param value: Object. The editor option value

    :return: The Javascript string fragment
    """
    option = JsUtils.jsConvertData(option, None)
    value = JsUtils.jsConvertData(value, None)
    return JsObjects.JsObjects.get("%s.setOption(%s, %s)" % (self.varId, option, value))

  def getOption(self):
    """
    Description:
    -----------
    Retrieves the current value of the given option for this editor instance.

    Related Pages:

			https://codemirror.net/doc/manual.html#api

    :return: The Javascript object
    """
    return JsObjects.JsObjects.get("%s.getOption()" % self.varId)

  def refresh(self):
    """
    Description:
    -----------
    If your code does something to change the size of the editor element (window resizes are already listened for), or unhides it, you should probably follow up by calling this method to ensure CodeMirror is still looking as intended. See also the autorefresh addon.

    Related Pages:

			https://codemirror.net/doc/manual.html#api

    :return: The Javascript string fragment
    """
    return self.fnc_closure("refresh()")

  def execCommand(self, command):
    """
    Description:
    -----------
    Runs the command with the given name on the editor.

    Related Pages:

			https://codemirror.net/doc/manual.html#commands

    :return: The Javascript string fragment
    """
    command = JsUtils.jsConvertData(command, None)
    return self.fnc_closure("execCommand(%s)" % command)

  def focus(self):
    """
    Description:
    -----------
    Give the editor focus.

    Related Pages:

			https://codemirror.net/doc/manual.html#api

    :return: The Javascript string fragment
    """
    return self.fnc_closure("focus()")



"""
Module dedicated to ensure the encoding between the two languages.

This will ensure a good data transfer between Python and Javascript.

This will rely on the standard json conversion but some extra checks are added in order to handle
some specific data structures in the Python languages.

For example Numpy and Pandas data structures (not correctly converted) are encoded thanks to this layer.
"""

import json
import datetime


class Encoder(json.JSONEncoder):
  """
  Python to Js Encoding

  Class in charge of encoding the data to be written on the Javascript side.
  In most of the function the simple json module is used but this module is there to encode more complex object
  frequently coming from Pandas

  Example
  >>> json.dumps({"test": ""}, cls=Encoder, allow_nan=False)
  '{"test": ""}'

  :return: A serializable item
  """
  def default(self, obj):
    try:
      import pandas

      if isinstance(obj, pandas.core.series.Series):
        return list(obj)

      elif isinstance(obj, datetime.datetime):
        if isinstance(obj, type(pandas.NaT)):
          return ''

        return obj.strftime('%Y-%m-%d')
    except: pass

    try:
      import numpy

      if isinstance(obj, (numpy.int_, numpy.intc, numpy.intp, numpy.int8, numpy.int16, numpy.int32, numpy.int64, numpy.uint8,
                          numpy.uint16, numpy.uint32, numpy.uint64, numpy.integer)):
        return int(obj)

      elif isinstance(obj, (numpy.float_, numpy.float16, numpy.float32, numpy.float64, numpy.floating)):
        return float(obj)

      elif isinstance(obj, numpy.ndarray):
        return obj.tolist()

    except: pass

    return super(Encoder, self).default(obj)





from epyk.core.html import tables as html_tables


class Datatables(object):
  def __init__(self, context):
    self.parent = context

  def table(self, records=None, cols=None, rows=None, width=(100, '%'), height=(None, 'px'), htmlCode=None,
            options=None, profile=None):
    """

    :param records:
    :param cols:
    :param rows:
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    data = []
    cols = cols or []
    rows = rows or []
    if not cols and not rows:
      cols = list(records[0].keys())
    for rec in records:
      data.append([rec.get(c) for c in cols + rows])

    table = html_tables.HtlmTableDatatable.Table(self.parent.context.rptObj, data, width, height, htmlCode, options, profile)
    for c in cols + rows:
      col_def = table.config.columns
      col_def.title = c
      col_def.className.center()
    table.style.themes.compact()
    self.parent.context.register(table)
    return table

  def heatmap(self):
    pass

  def intensity(self):
    pass

  def hierarchy(self):
    pass

  def delta_signed(self):
    pass

  def delta_abs(self):
    pass

  def comments(self):
    pass




import sys

from epyk.core.html import graph


class Vis(object):
  def __init__(self, context):
    self.parent = context
    self.chartFamily = "Vis"

  def line(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None,
           htmlCode=None):
    """
    Description:
    -----------
    Graph2d is an interactive visualization chart to draw data in a 2D graph. You can freely move and zoom in the graph by dragging and scrolling in the window.

    Graph2d uses HTML DOM and SVG for rendering. This allows for flexible customization using css styling.

    elated Pages:
    --------------
    http://www.chartjs.org/
    https://visjs.github.io/vis-timeline/examples/graph2d/16_bothAxisTitles.html

    Attributes:
    ----------
    :param record:
    :param y_columns:
    :param x_axis:
    :param profile:
    :param height:
    :param htmlCode:
    """
    agg_data = {}
    for rec in record:
      for y in y_columns:
        if y in rec:
          agg_data.setdefault(y, {})[rec[x_axis]] = agg_data.get(y, {}).get(rec[x_axis],  0) + float(rec[y])
    labels, data = set(), []
    for i, c in enumerate(y_columns):
      series = []
      for x, y in agg_data[c].items():
        labels.add(x)
        series.append({"x": x, "y": y, 'group': i})
      data.append(series)

    line_chart = graph.GraphVis.ChartLine(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    #line_chart.groups.add(sorted(list(labels)))
    for d in data:
      line_chart.add_items(d)

    self.parent.context.register(line_chart)
    return line_chart

  def scatter(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"),
              options=None, htmlCode=None):
    """
    Description:
    -----------

    elated Pages:
    --------------
    http://www.chartjs.org/

    Attributes:
    ----------
    :param record:
    :param y_columns:
    :param x_axis:
    :param profile:
    :param height:
    :param htmlCode:
    """
    agg_data = {}
    for rec in record:
      for y in y_columns:
        if y in rec:
          agg_data.setdefault(y, {})[rec[x_axis]] = agg_data.get(y, {}).get(rec[x_axis],  0) + float(rec[y])
    labels, data = set(), []
    for i, c in enumerate(y_columns):
      series = []
      for x, y in agg_data[c].items():
        labels.add(x)
        series.append({"x": x, "y": y, 'group': i})
      data.append(series)

    line_chart = graph.GraphVis.ChartScatter(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.options.height = height[0]
    #line_chart.groups.add(sorted(list(labels)))
    for d in data:
      line_chart.add_items(d)

    self.parent.context.register(line_chart)
    return line_chart

  def bar(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"),
          options=None, htmlCode=None):
    """
    Description:
    -----------

    elated Pages:
    --------------
    http://www.chartjs.org/

    Attributes:
    ----------
    :param record:
    :param y_columns:
    :param x_axis:
    :param profile:
    :param height:
    :param htmlCode:
    """
    agg_data = {}
    for rec in record:
      for y in y_columns:
        if y in rec:
          agg_data.setdefault(y, {})[rec[x_axis]] = agg_data.get(y, {}).get(rec[x_axis],  0) + float(rec[y])
    labels, data = set(), []
    for i, c in enumerate(y_columns):
      series = []
      for x, y in agg_data[c].items():
        labels.add(x)
        series.append({"x": x, "y": y, 'group': i})
      data.append(series)

    line_chart = graph.GraphVis.ChartBar(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.options.height = height[0]
    #line_chart.groups.add(sorted(list(labels)))
    for d in data:
      line_chart.add_items(d)

    self.parent.context.register(line_chart)
    return line_chart

  def surface(self, record, y_columns=None, x_axis=None, z_axis=None, profile=None, width=(100, "%"),
              height=(330, "px"), options=None, htmlCode=None):
    agg_data = {}
    for rec in record:
      key_point = (rec[x_axis], rec[z_axis])
      for y in y_columns:
        if y in rec:
          agg_data.setdefault(y, {})[key_point] = agg_data.get(y, {}).get(key_point,  0) + float(rec[y])
    labels, data = set(), []
    for i, c in enumerate(y_columns):
      series = []
      for point, y in agg_data[c].items():
        series.append({"x": point[0], "y": y, 'z': point[1], 'group': i})
      data.append(series)

    line_chart = graph.GraphVis.Chart3D(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    for d in data:
      line_chart.add_items(d)

    self.parent.context.register(line_chart)
    return line_chart

  def scatter3d(self, record, y_columns=None, x_axis=None, z_axis=None, profile=None, width=(100, "%"),
                height=(330, "px"), options=None, htmlCode=None):
    agg_data = {}
    for rec in record:
      key_point = (rec[x_axis], rec[z_axis])
      for y in y_columns:
        if y in rec:
          agg_data.setdefault(y, {})[key_point] = agg_data.get(y, {}).get(key_point,  0) + float(rec[y])
    labels, data = set(), []
    for i, c in enumerate(y_columns):
      series = []
      for point, y in agg_data[c].items():
        series.append({"x": point[0], "y": y, 'z': point[1], 'group': i})
      data.append(series)

    line_chart = graph.GraphVis.Chart3DScatter(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    for d in data:
      line_chart.add_items(d)

    self.parent.context.register(line_chart)
    return line_chart

  def series3d(self, aresDf=None, seriesNames=None, xAxis=None, otherDims=None, dataFncs=None, title='',
           globalFilter=None, filterSensitive=True, profile=None, dataSrc=None, xAxisOrder=None, chartOptions=None,
           width=100, widthUnit="%", height=330, heightUnit="px", htmlCode=None):
    """

    :param aresDf:
    :param seriesNames:
    :param xAxis:
    :param otherDims:
    :param dataFncs:
    :param title:
    :param globalFilter:
    :param filterSensitive:
    :param profile:
    :param dataSrc:
    :param xAxisOrder:
    :param chartOptions:
    :param width:
    :param widthUnit:
    :param height:
    :param heightUnit:
    :param htmlCode:
    :return:
    :rtype: graph.GraphVis.Chart
    """
    return self.parent.context.chart(chartType=sys._getframe().f_code.co_name, aresDf=aresDf, seriesNames=seriesNames,
                                     xAxis=xAxis, otherDims=otherDims, dataFncs=dataFncs, title=title, chartFamily=self.chartFamily,
                                     globalFilter=globalFilter, filterSensitive=filterSensitive, profile=profile, dataSrc=dataSrc,
                                     xAxisOrder=xAxisOrder, chartOptions=chartOptions, width=width, widthUnit=widthUnit,
                                     height=height, heightUnit=heightUnit, htmlCode=htmlCode)

  def line3d(self, record, y_columns=None, x_axis=None, z_axis=None, profile=None, width=(100, "%"),
                height=(330, "px"), options=None, htmlCode=None):
    agg_data = {}
    for rec in record:
      key_point = (rec[x_axis], rec[z_axis])
      for y in y_columns:
        if y in rec:
          agg_data.setdefault(y, {})[key_point] = agg_data.get(y, {}).get(key_point,  0) + float(rec[y])
    labels, data = set(), []
    for i, c in enumerate(y_columns):
      series = []
      for point, y in agg_data[c].items():
        series.append({"x": point[0], "y": y, 'z': point[1], 'group': i})
      data.append(series)

    line_chart = graph.GraphVis.Chart3DLine(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    for d in data:
      line_chart.add_items(d)

    self.parent.context.register(line_chart)
    return line_chart

  def bar3d(self, record, y_columns=None, x_axis=None, z_axis=None, profile=None, width=(100, "%"),
                height=(330, "px"), options=None, htmlCode=None):
    agg_data = {}
    for rec in record:
      key_point = (rec[x_axis], rec[z_axis])
      for y in y_columns:
        if y in rec:
          agg_data.setdefault(y, {})[key_point] = agg_data.get(y, {}).get(key_point,  0) + float(rec[y])
    labels, data = set(), []
    for i, c in enumerate(y_columns):
      series = []
      for point, y in agg_data[c].items():
        series.append({"x": point[0], "y": y, 'z': point[1], 'group': i})
      data.append(series)

    line_chart = graph.GraphVis.Chart3DBar(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    for d in data:
      line_chart.add_items(d)

    self.parent.context.register(line_chart)
    return line_chart

  def timeline(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"),
               options=None, htmlCode=None):
    """
    Description:
    -----------

    Related Pages:
http://www.chartjs.org/

    Attributes:
    ----------
    :param record:
    :param y_columns:
    :param x_axis:
    :param profile:
    :param height:
    :param htmlCode:
    """
    series = []
    for rec in record:
      for i, y in enumerate(y_columns):
        if y in rec:
          series.append({"x": rec[x_axis], "y": rec[y], 'group': i})

    line_chart = graph.GraphVis.ChartTimeline(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.options.height = height[0]
    line_chart.options.editable = True
    line_chart.options.showCurrentTime = True
    line_chart.add_items(series)

    self.parent.context.register(line_chart)
    return line_chart

  def network(self, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    line_chart = graph.GraphVis.ChartNetwork(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.options.height = height[0]
    self.parent.context.register(line_chart)
    return line_chart





from epyk.core.data import DataClass


class FixedHeater(DataClass):

  def activate(self):
    """

    :return:
    """
    self.header = True
    return self

  @property
  def header(self):
    return self._attrs["header"]

  @header.setter
  def header(self, val):
    self._attrs["header"] = val

  @property
  def headerOffset(self):
    return self._attrs["header"]

  @headerOffset.setter
  def headerOffset(self, val):
    self._attrs["headerOffset"] = val

  @property
  def footer(self):
    return self._attrs["footer"]

  @footer.setter
  def footer(self, val):
    self._attrs["footer"] = val

  @property
  def footerOffset(self):
    return self._attrs["footerOffset"]

  @footerOffset.setter
  def footerOffset(self, val):
    self._attrs["footerOffset"] = val



"""
Entry point for all the Media components.
"""

from epyk.core import html


class Media(object):
  def __init__(self, context):
    self.context = context

  def video(self, value, path=None, width=(100, '%'), height=(None, 'px'), htmlCode=None, profile=None, options=None):
    """
    Add a video from the server to the page.
    The format for the video must be MP4

    Usage::

      rptObj.ui.media.video("CWWB3673.MP4")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMedia.Media`

    Related Pages:

        https://www.w3schools.com/html/html5_video.asp

    Attributes:
    ----------
    :param value: The name of the video
    :param path: Optional. THe path to the video
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. The component identifier code (for both Python and Javascript)
    :param profile: Optional. A flag to set the component performance storage
    :param options: Optional. A dictionary with the components properties
    """
    dft_options = {"autoplay": True}
    if options is not None:
      dft_options.update(options)
    html_media = html.HtmlMedia.Media(self.context.rptObj, value, path, width, height, htmlCode, profile, dft_options)
    self.context.register(html_media)
    return html_media

  def audio(self, value, path=None, width=(100, '%'), height=(None, 'px'), htmlCode=None, profile=None, options=None):
    """
    Add a audio track from the server to the page.
    The format for the video must be mpeg

    Usage::

      rptObj.ui.media.video("CWWB3673.mpeg")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMedia.Audio`

    Related Pages

    https://www.w3schools.com/html/html5_video.asp

    Attributes:
    ----------
    :param value: The name of the audio object
    :param path: Optional. THe path to the audio object
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. The component identifier code (for both Python and Javascript)
    :param profile: Optional. A flag to set the component performance storage
    :param options: Optional. A dictionary with the components properties
    """
    dft_options = {"autoplay": True}
    if options is not None:
      dft_options.update(options)
    html_audio = html.HtmlMedia.Audio(self.context.rptObj, value, path, width, height, htmlCode, profile, dft_options)
    self.context.register(html_audio)
    return html_audio

  def youtube(self, link, width=(100, '%'), height=(None, 'px'), htmlCode=None, profile=None, options=None):
    """
    This will add a youtube video using the shared line to embedded to a website.

    Usage::

      rptObj.ui.media.youtube("https://www.youtube.com/embed/dfiHMtih5Ac")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMedia.Youtube`

    Related Pages

    https://www.w3schools.com/html/html5_video.asp

    Attributes:
    ----------
    :param link: The youtube link
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. The component identifier code (for both Python and Javascript)
    :param profile: Optional. A flag to set the component performance storage
    :param options: Optional. A dictionary with the components properties
    """
    dflt_options = {"width": "420", "height": "315", "type": "text/html"}
    if options is not None:
      dflt_options.update(options)
    html_youtube = html.HtmlMedia.Youtube(self.context.rptObj, link, width, height, htmlCode, profile, dflt_options)
    self.context.register(html_youtube)
    return html_youtube

  # def camera(self):
  #   """
  #
  #   Documentation
  #   https://www.html5rocks.com/en/tutorials/getusermedia/intro/
  #
  #   Attributes:
  #   ----------
  #   :return:
  #   """



"""
CSS Style module for the Tabs components
"""

from epyk.core.css.styles.classes import CssStyle


class CssDefaultTab(CssStyle.Style):
  _attrs = {'margin': 0}

  def customize(self):
    self.css({'background-color': self.rptObj.theme.greys[0], 'color': self.rptObj.theme.colors[-1]})


class CssDefaultTabSelected(CssStyle.Style):

  def customize(self):
    self.css({'background-color': self.rptObj.theme.colors[-1], 'color': self.rptObj.theme.colors[0]})


class CssBorderTab(CssStyle.Style):
  _attrs = {'margin': '0 0 5px 0'}

  def customize(self):
    self.css({'border-bottom': 'None', 'background-color': self.rptObj.theme.greys[0],
              'color': self.rptObj.theme.colors[-1]})


class CssBorderTabSelected(CssStyle.Style):

  def customize(self):
    self.css({'border-bottom': "3px solid %s" % self.rptObj.theme.success[1]}, important=True)




from epyk.core.html import Html
from epyk.core.html.options import OptChartJs

from epyk.core.js import JsUtils
from epyk.core.js.primitives import JsObject

from epyk.core.js.packages import JsChartJs
from epyk.core.js.packages import JsD3


class Chart(Html.Html):
  name, category, callFnc = 'ChartJs', 'Charts', 'chartJs'

  def __init__(self,  report, width, height, htmlCode, options, profile):
    self.height = height[0]
    super(Chart, self).__init__(report, [], code=htmlCode, css_attrs={"width": width, "height": height}, profile=profile)
    self._d3, self._chart, self._datasets, self._options, self._data_attrs, self._attrs = None, None, [], None, {}, {}
    self._options_init = options
    self.style.css.margin_top = 10

  @property
  def chartId(self):
    """
    Return the Javascript variable of the chart
    """
    return "%s_obj" % self.htmlId

  @property
  def d3(self):
    if self._d3 is None:
      self._d3 = JsD3.D3Select(self._report, id="#%s" % self.htmlId)
    return self._d3

  @property
  def options(self):
    """

    :rtype: Options
    :return:
    """
    if self._options is None:
      self._options = OptChartJs.Options(self._report, attrs=self._options_init)
    return self._options

  def labels(self, labels):
    """

    :param labels:
    :return:
    """
    self._data_attrs['labels'] = labels
    return self

  def dataset(self, i=None):
    """

    :rtype: JsChartJs.DataSetPie
    """
    if i is None:
      return self._datasets[-1]

    return self._datasets[i]

  @property
  def datasets(self):
    return self._datasets

  def getCtx(self):
    obj_datasets = "[%s]" % ", ".join([d.toStr() for d in self._datasets])
    self._data_attrs['datasets'] = JsObject.JsObject.get(obj_datasets)
    obj_data = "{%s}" % ", ".join(["%s: %s" % (k, JsUtils.jsConvertData(v, None)) for k, v in self._data_attrs.items()])
    self._attrs["data"] = JsObject.JsObject.get(obj_data)
    self._attrs["options"] = JsObject.JsObject.get(str(self.options))
    str_ctx = "{%s}" % ", ".join(["%s: %s" % (k, JsUtils.jsConvertData(v, None)) for k, v in self._attrs.items()])
    return str_ctx

  def build(self, data=None, options=None, profile=False):
    return '%s = new Chart(%s.getContext("2d"), %s)' % (self.chartId, self.dom.varId, self.getCtx())

  def __str__(self):
    self._report._props.setdefault('js', {}).setdefault("builders", []).append(self.refresh())
    return '<canvas %s></canvas>' % self.get_attrs(pyClassNames=self.style.get_classes())


class Datasets(object):

  def __init__(self, report):
    self._report, self.__data = report, []

  def add(self, data):
    self.__data.append(data)
    return self


class ChartLine(Chart):
  __reqJs = ['Chart.js']

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(ChartLine, self).__init__(report, width, height, htmlCode, options, profile)
    self._attrs['type'] = 'line'

  @property
  def options(self):
    """

    :rtype: OptionsPolar
    :return:
    """
    if self._options is None:
      self._options = OptChartJs.OptionsLine(self._report, attrs=self._options_init)
    return self._options

  def add_dataset(self, data, colors=None, opacity=None):
    """

    :return:
    """
    data = JsChartJs.DataSetScatterLine(self._report, attrs={"data": data})
    data.fill = False
    data.label = self._data_attrs['labels'][len(self._datasets)]
    if colors is None:
      data.borderColor = self._report.theme.charts[len(self._datasets)]
      data.backgroundColor = self._report.theme.charts[len(self._datasets)]
    data.borderWidth = 1
    data.pointRadius = 1
    if opacity is not None:
      data.fillOpacity = opacity
    self._datasets.append(data)
    return data


class ChartBubble(Chart):
  __reqJs = ['Chart.js']

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(ChartBubble, self).__init__(report, width, height, htmlCode, options, profile)
    self._attrs['type'] = 'bubble'

  def add_dataset(self, data, colors=None, opacity=0.8):
    """

    """
    data = JsChartJs.DataSetBubble(self._report, attrs={"data": data})
    data.fill = False
    data.label = self._data_attrs['labels'][len(self._datasets)]
    if colors is None:
      data.borderColor = self._report.theme.charts[len(self._datasets)]
      data.backgroundColor = self._report.theme.charts[len(self._datasets)]
      data.fillOpacity = opacity
    self._datasets.append(data)
    return data


class ChartBar(Chart):
  __reqJs = ['Chart.js']

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(ChartBar, self).__init__(report, width, height, htmlCode, options, profile)
    self._attrs['type'] = 'bar'

  @property
  def options(self):
    """

    :rtype: OptionsPolar
    :return:
    """
    if self._options is None:
      self._options = OptChartJs.OptionsBar(self._report, attrs=self._options_init)
    return self._options

  def add_dataset(self, data, type=None, colors=None, opacity=0.8):
    """

    :return:
    """
    if type is not None:
      data = JsChartJs.DataSetBar(self._report, attrs={"data": data, 'type': type})
    else:
      data = JsChartJs.DataSetBar(self._report, attrs={"data": data})
    self._datasets.append(data)
    data.label = self._data_attrs['labels'][len(self._datasets)]
    if colors is None:
      data.backgroundColor = self._report.theme.charts[len(self._datasets)]
      data.fillOpacity = opacity
    return data


class ChartPolar(Chart):
  __reqJs = ['Chart.js']

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(ChartPolar, self).__init__(report, width, height, htmlCode, options, profile)
    self._attrs['type'] = 'polarArea'

  @property
  def options(self):
    """

    :rtype: OptionsPolar
    :return:
    """
    if self._options is None:
      self._options = OptChartJs.OptionsPolar(self._report, attrs=self._options_init)
    return self._options

  def add_dataset(self, data, type=None, colors=None, opacity=0.4):
    """

    :return:
    """
    if type is not None:
      data = JsChartJs.DataSetPolar(self._report, attrs={"data": data, 'type': type})
    else:
      data = JsChartJs.DataSetPolar(self._report, attrs={"data": data})
    self._datasets.append(data)
    data.label = self._data_attrs['labels'][len(self._datasets)]
    #if colors is None:
    data.backgroundColor = self._report.theme.charts
    data.fillOpacity = opacity
    return data


class ChartHBar(ChartBar):
  __reqJs = ['Chart.js']

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(ChartBar, self).__init__(report, width, height, htmlCode, options, profile)
    self._attrs['type'] = 'horizontalBar'


class ChartPie(Chart):
  __reqJs = ['Chart.js']

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(ChartPie, self).__init__(report, width, height, htmlCode, options, profile)
    self._attrs['type'] = 'pie'

  @property
  def options(self):
    """

    :rtype: OptionsPie
    :return:
    """
    if self._options is None:
      self._options = OptChartJs.OptionsPie(self._report, attrs=self._options_init)
    return self._options

  def add_dataset(self, data, colors=None, opacity=0.8):
    """

    :return:
    """
    data = JsChartJs.DataSetPie(self._report, attrs={"data": data})
    self._datasets.append(data)
    if colors is None:
      data.backgroundColor = self._report.theme.charts
      data.fillOpacity = opacity
    return data


class ChartRadar(Chart):
  __reqJs = ['Chart.js']

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(ChartRadar, self).__init__(report, width, height, htmlCode, options, profile)
    self._attrs['type'] = 'radar'

  def add_dataset(self, data, colors=None, opacity=0.4):
    """

    :return:
    """
    data = JsChartJs.DataSetRadar(self._report, attrs={"data": data})
    self._datasets.append(data)
    if colors is None:
      data.backgroundColor = self._report.theme.charts[len(self._datasets)]
      data.borderColor = self._report.theme.charts[len(self._datasets)]
      data.borderWidth = 0.2
      data.fillOpacity = opacity
    return data


class ChartScatter(Chart):
  __reqJs = ['Chart.js']

  def __init__(self, report, width, height, htmlCode, options, profile):
    super(ChartScatter, self).__init__(report, width, height, htmlCode, options, profile)
    self._attrs['type'] = 'scatter'

  def add_dataset(self, data, colors=None):
    """

    :return:
    """
    data = JsChartJs.DataSetScatterLine(self._report, attrs={"data": data})
    data.fill = False
    data.label = self._data_attrs['labels'][len(self._datasets)]
    if colors is None:
      data.backgroundColor = self._report.theme.charts[len(self._datasets)]
      data.borderColor = self._report.theme.charts[len(self._datasets)]
    self._datasets.append(data)
    return data




from epyk.core.html.options import Options
from epyk.core.js.packages import packageImport


class OptionsText(Options):

  @property
  def reset(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return self._config_get(False)

  @reset.setter
  def reset(self, bool):
    self._config(bool)

  @property
  def markdown(self):
    """
    Description:
    ------------
    Showdown is a Javascript Markdown to HTML converter, based on the original works by John Gruber.
    Showdown can be used client side (in the browser) or server side (with NodeJs).

    Related Pages:

			https://github.com/showdownjs/showdown
    """
    return self._config_get(False, 'showdown')

  @markdown.setter
  @packageImport("showdown")
  def markdown(self, values):
    values = {} if values is True else values
    self._config(values, 'showdown')

  @property
  def showdown(self):
    """
    Description:
    ------------
    Showdown is a Javascript Markdown to HTML converter, based on the original works by John Gruber.
    Showdown can be used client side (in the browser) or server side (with NodeJs).

    Related Pages:

			https://github.com/showdownjs/showdown
    """
    return self._config_get(False)

  @showdown.setter
  @packageImport("showdown")
  def showdown(self, values):
    values = {} if values is True else values
    self._config(values)

  @property
  def limit_char(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return self._config_get(None, 'limit_char')

  @limit_char.setter
  def limit_char(self, value):
    self._config(value, "maxlength")

  @property
  def red(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return self._config_get(self._report.theme.danger[1])

  @red.setter
  def red(self, value):
    self._config(value)

  @property
  def green(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return self._config_get(self._report.theme.success[1])

  @green.setter
  def green(self, value):
    self._config(value)

  @property
  def orange(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return self._config_get(self._report.theme.warning[1])

  @orange.setter
  def orange(self, value):
    self._config(value)

  @property
  def font_size(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return self._config_get('none')

  @font_size.setter
  def font_size(self, value):
    self._config(value)


class OptionsTitle(OptionsText):

  @property
  def content_table(self):
    """
    Description:
    ------------

    Related Pages:
"""
    return self._config_get(True)

  @content_table.setter
  def content_table(self, bool):
    self._config(bool)


class OptionsNumber(OptionsText):

  @property
  def digits(self):
    """
    Description:
    ------------
    decimal point separator

    Related Pages:
http://openexchangerates.github.io/accounting.js/
    """
    return self._config_get(0)

  @digits.setter
  def digits(self, num):
    self._config(num)

  @property
  def format(self):
    """
    Description:
    ------------
    controls output: %s = symbol, %v = value/number

    Related Pages:
http://openexchangerates.github.io/accounting.js/
    """
    return self._config_get("%s%v")

  @format.setter
  def format(self, num):
    self._config(num)

  @property
  def symbol(self):
    """
    Description:
    ------------
    default currency symbol is ''

    Related Pages:
http://openexchangerates.github.io/accounting.js/#documentation
    """
    return self._config_get("")

  @symbol.setter
  def symbol(self, value):
    self._report._jsStyles["type_number"] = "money"
    self._config(value)

  @property
  def thousand_sep(self):
    """
    Description:
    ------------
    thousands separator

    Related Pages:
http://openexchangerates.github.io/accounting.js/
    """
    return self._config_get(",")

  @thousand_sep.setter
  def thousand_sep(self, value):
    self._config(value)

  @property
  def decimal_sep(self):
    """
    Description:
    ------------
    decimal point separator

    Related Pages:
http://openexchangerates.github.io/accounting.js/
    """
    return self._config_get(".")

  @decimal_sep.setter
  def decimal_sep(self, value):
    self._config(value)


class OptionsConsole(OptionsText):

  @property
  def timestamp(self):
    """
    Description:
    ------------
    """
    return self._attrs.get('timestamp', False)

  @timestamp.setter
  def timestamp(self, bool):
    self.set(bool)


class OptionsComposite(Options):

  @property
  def reset_class(self):
    """
    Description:
    ------------
    """
    return self._attrs.get('reset_class', False)

  @reset_class.setter
  def reset_class(self, bool):
    self.set(bool)
